# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module coinparsers
##   Classes de parsing (minihtml, minixml, horloges...)
## ----------------------------------------------------------------------

import re
from htmlentitydefs import entitydefs
from urllib import quote, quote_plus

import logging
log = logging.getLogger('pycc')

# ----------------------------------------------------------
# Gestion Unicode
# ----------------------------------------------------------
import types
_unicode = types.UnicodeType

def toUnicode(text, encoding='iso-8859-1'):
  if type(text) == _unicode:
    return text
  else:
    try:
      return text.decode(encoding)
    except:
      pass
    log.warning("Chaine unicode pourrite: "+repr(text))
    try:
      return unicode(text, "utf-8")
    except:
      pass
    log.warning("Chaine doublement pourrite")
    return unicode(repr(text), encoding)


# ================================
# Gestion des horloges-rfrences
# ================================

class ClockParser:
  """ Cette classe sert  parser les horloges-rfrences.
      C'est un binz pas possible mais la regexp fait le gros du boulot.
  """
  def __init__(self):
    # Expression rgulire 'horloge' pour reprer les rfrences aux posts
    self.re_clock = re.compile("^(\d\d?)(?::|[.]|h)?(\d\d)(?::|[.]|m)?(?:(\d\d)(?:s|(?:[:^]?([0-9])))?)?([^0-9])*$")
  
  def parse(self, cstring, cur_trib):
    """ Parse une chane et renvoie la liste des horloges-rfrences trouves
    """
    bufclock = ur""
    cpt = 0
    cstring += ur" "
    clocks = []
    for ch in cstring:
      cpt += 1
      if ch in ur"0123456789" or (bufclock and (bufclock[-1] in ur"0123456789" and (ch in ur":.hms^"))):
        bufclock = bufclock + ch
      else:
        if len(bufclock) >= 4:
          marks = self.test(bufclock, cstring, cpt, cur_trib)
          if marks is not None:
            clocks.append(marks)
        bufclock = ur""
        continue
    return clocks
  
  def test(self, clockstring, strref, cpt, cur_trib):
    """ Taiste si une chane contient une norloge.
        Tous les paramtres sont incomprhensibles mais indispensables.
    """
    ## en premier, taiste la regexp
    clock = self.re_clock.match(clockstring)
    if not clock: return None
    
    ## la regexp a fonctionn, on calcule les bornes dans la string
    refstring = strref[:cpt]
    beg_ref = len(refstring)-len(clockstring)-1
    time_tup = clock.groups()
    end_ref = clock.end()
    if time_tup[4]: end_ref = end_ref - len(time_tup[4])
    
    ## on extrait les h, m, s et tout le toutim
    h = time_tup[0]
    if int(h) > 23: return None
    if len(h) == 1: h = '0'+h
    m = time_tup[1]
    if int(m) > 59: return None
    s = time_tup[2]
    if s is None: s = '--'
    else:
      if int(s) > 59: return None
    try:
      n = time_tup[3].encode('iso-8859-15')
    except:
      n = time_tup[3] ## en cas de problme unicode
    if n is None: num = 0
    elif n == '': num = 1
    elif n == '': num = 2
    elif n == '': num = 3
    else:
      num = int(n)
      if num == 0: num = 1
    
    ## pour les tribunes anglophones (boulays...)
    ampm = strref[cpt-1:cpt+1].lower()
    if ampm == "pm":
      h = str(int(h)+12)
    if ampm == "pm" or ampm == "am":
      cpt += 2
      end_ref += 2
    
    ## et zyva dans la cross-rfrence
    trib = ""
    if strref[cpt-1] == "@":
      for ch in strref[cpt:]:
        if ch.lower() not in "abcdefghijklmnopqrstuvwxyz0123456789_-":
          break
        trib += ch
    if not trib:
      trib = cur_trib
    else:
      end_ref += len(trib)+1
    
    ## ay, on peut renvoyer le bouzin
    ref = "ref%s%s%s%s" % (h, m, s, num)
    marks = (beg_ref, beg_ref+end_ref, ref, trib)
    return marks
  

# ================================
# Gestion du xml
# ================================

class MiniXmlError(Exception):
  def __init__(self, msg=""):
    self.message = msg
  def getMessage(self):
    return self.message
  def __str__(self):
    return "MiniXml Parser: " + self.getMessage()

def getLine(fh):
  line = fh.readline()
  return zapEntities(zapComments(line))

def getAttr(name, balise):
  """ Renvoie la valeur de l'attribut 'name' dans la balise 'balise'
  """
  start = balise.find(name+'="')
  if start >= 0:
    start += len(name)+2
    end = balise.find('"', start)
    return balise[start:end]
  start = balise.find(name+"='")
  if start >= 0:
    start += len(name)+2
    end = balise.find("'", start)
    return balise[start:end]
  raise MiniXmlError("%s: attribut introuvable" % name)
  
def getContent(balise, str):
  """ Renvoie le contenu de la balise 'balise' dans la chane 'str'
  """
  start = str.find("<"+balise)
  if start >= 0:
    start = str.find(">", start)
    if str[start-1] == "/":
      return "" # Balise vide ("<balise/>")
    end = str.find("</"+balise, start)
    if end > start:
      return str[start+1:end].strip()
    else:
      raise MiniXmlError("%s: balise pas referme" % balise)
  else:
    raise MiniXmlError("%s: balise introuvable" % balise)
  
def zapEntities(msg, encoding=None):
  """ Remplace dans la chane 'msg' les entits xml par leur valeur
  """
  start = msg.find("&")
  end = 0
  res = ""
  while start >= 0:
    res += msg[end:start]
    end = msg.find(";", start)
    if end < start:
      end = start
      break
    entity = msg[start+1:end]
    if entitydefs.has_key(entity):
      if encoding is not None:
        res += unicode(entitydefs[entity], encoding)
      else:
        res += entitydefs[entity]
    else:
      try:
        char = "?%s?" % entity
        if entity[0] == "#":
          if entity[1].lower() == 'x':
            code = int(entity[2:], 16)
          else:
            code = int(entity[1:])
          char = eval('u"\u%04x"' % code)
      except:
        pass
      res += char
    end += 1
    start = msg.find("&", end)
  res += msg[end:]
  return res
  
def zapComments(msg):
  """ Renvoie la chane 'msg' purge des commentaires xml
  """
  start = msg.find("<!--")
  end = 0
  res = ""
  while start >= 0:
    res += msg[end:start]
    end = msg.find("-->", start)
    end += 3
    start = msg.find("<!--", end)
  res += msg[end:]
  return res
  

# ================================
# Gestion du mini-html des posts
# ================================

""" Les entits de base """
basentities = { 'lt': '', 'gt': '', 'amp': '&', 'quot': '"' }
# L'oeil avis remarquera que lt et gt ne sont pas des vrais < et > ici, mais des guillemets simples
# C'est gruiiik mais c'est la meilleure faon que j'ai trouve pour ne pas qu'ils soient confondus
# avec des tags pendant le parsing, tout en conservant la bonne longueur  la chane.
# Les bons caractres sont rtablis  la fin du parsing.


class MiniHtmlParser:
  """ Cette classe est un parser pour l'html minimal des posts de la
      tribune. Il reconnait les tags dfinis dans valid_tags.
  """
  def __init__(self):
    self.encoding = 'iso-8859-15'
    self.marks = {}
    # La liste des tags autoriss par dfaut:
    self.valid_tags = [ 'a', 'b', 'i', 's', 'tt', 'u', 'span', 'img' ]
  
  def _parse(self, htmlstring, balise='', index=0, res_index=0):
    """ Fonction rcursive interne parsant un bloc de texte entre 2 balises
    """
    res = ""
    beg = index
    index = htmlstring.find("<", beg)
    while index >= beg:
      end = htmlstring.find(">", index)
      nnend = htmlstring.find("<", index+1)
      if nnend > index and nnend < end:
        log.debug("MiniHtml: deux '<' ouvrants de suite dtects")
        index = nnend
        end = htmlstring.find(">", index)
      if index > beg:
        res += htmlstring[beg:index]
        res_index += (index-beg)
      if end < index:
        tag = htmlstring[index:]
        log.info("MiniHtml: Balise pas referme: %s..." % repr(tag))
        return res+tag, len(htmlstring)
      tag = htmlstring[index+1:end]
      index = end+1
      if tag != "":
        if tag[0] == "/":
          tag = tag.lower()
          if balise != '':
            if tag[1:] == balise:
              return res, index
          log.warning("MiniHtml: Balise fermante %s trouve aprs l'ouvrante %s" % (repr(tag), repr(balise)))
          res += "<"+tag+">"
          res_index += len(tag)+2
        else:
          url = ""
          if tag[:7] == 'a href=':
            url = tag[8:-1]
            url = url.replace("&amp;", "&")
            tag = 'a'
          elif tag[:4] == 'span' and len(tag) > 28:
            url = tag[29]
            tag = 'span'
          tag = tag.lower()
          if tag in self.valid_tags:
            tag_beg = res_index
            parsed, index = self._parse(htmlstring, tag, index, res_index)
            res += parsed
            res_index += len(parsed)
            tag_marks = (tag_beg, res_index)
            if tag == 'span':
              tag=url                # gruik
              if tag == 'l': tag='s' # gruiiiiik
            if tag == 'pre': tag='tt'
            try:
              self.marks[tag].append(tag_marks)
            except KeyError:
              self.marks[tag] = [tag_marks]
            if tag == 'a':
              ref = "url_"+url
              try:
                self.marks[ref].append(tag_marks)
              except KeyError:
                self.marks[ref] = [tag_marks]
          else:
            log.warning("MiniHtml: Balise invalide ou mal place: %s" % repr(tag))
            res += "<"+tag+">"
            res_index += len(tag)+2
      beg = index
      index = htmlstring.find("<", beg)
    
    return res + htmlstring[beg:], len(htmlstring)
  
  def pre_parse(self, htmltext, slip_style="Tags not encoded", encoding=None):
    if encoding is not None:
      self.encoding = encoding
    inputstring = toUnicode(htmltext, self.encoding)
    if slip_style == "Tags not encoded":
      # Remplacement par des faux < > pour ne pas confondre
      inputstring = inputstring.replace('&lt;', u'')
      inputstring = inputstring.replace('&gt;', u'')
    inputstring = zapEntities(inputstring, self.encoding)
    if slip_style == "Tags encoded":
      # Remplacement par des faux < > pour ne pas confondre
      inputstring = inputstring.replace('&lt;', u'')
      inputstring = inputstring.replace('&gt;', u'')
      inputstring = inputstring.replace('&amp;', u'&')
    elif slip_style == "Hybride":
      inputstring = inputstring.replace('&lt;', u'<')
      inputstring = inputstring.replace('&gt;', u'>')
      inputstring = inputstring.replace('&amp;', u'&')
    try:
      inputstring = inputstring.encode(self.encoding)
    except:
      # print "--- prout sur outputstring", type(inputstring)
      pass
    return inputstring
  
  def html_parse(self, htmltext, tags=None, encoding=None):
    text = htmltext.replace("<br>", "\n")
    text = text.replace("<br/>", "\n")
    text = text.replace("<br />", "\n")
    if tags is None:
      tags = [ 'p', 'table', 'tr', 'th', 'td', 'h1', 'h2', 'h3', 'h4', 'div' ]
    old_vtags = self.valid_tags
    self.valid_tags += tags
    text = text.replace("<em>", "<i>")
    text = text.replace("</em>", "</i>")
    text = text.replace("<strong>", "<b>")
    text = text.replace("</strong>", "</b>")
    text = text.replace("<pre>", "<tt>")
    text = text.replace("</pre>", "</tt>")
    text = text.replace("</p>", "\n</p>")
    text = text.replace("</div>", "\n</div>")
    msg, marks = self.parse(text, encoding=encoding)
    self.valid_tags = old_vtags
    return msg, marks
  
  def parse(self, htmltext, encoding=None):
    """ Prend un texte en entre et renvoie la chane parse et une
        liste d'lments de la forme 'tag: marqueurs', o 'marqueurs'
        est elle-mme une liste de couples (debut, fin)
    """
    if encoding is not None:
      self.encoding = encoding
    inputstring = toUnicode(htmltext, self.encoding)
    self.marks = {}
    outputstring, lg = self._parse(inputstring)
    outputstring = outputstring.replace(u'', '<') # Restauration des vrais < et > (oui c'est crade,
    outputstring = outputstring.replace(u'', '>') # cf. remarque plus haut sur basentities)
    return outputstring, self.marks
  

def parseWikiUrls(str, server, marks):
  """ Fonction pour parser les url-wiki
  """
  start = str.find("[")
  while start >= 0:
    end = str.find("]", start)
    if end > start:
      itemeuh = str[start+1:end]
      try:
        item = itemeuh.encode("iso-8859-15")
      except:
        item = itemeuh
      urlreflist = []
      for tag in marks.keys():
        if tag[:4] == "url_": urlreflist += [ mark[0] for mark in marks[tag] ]
      tag_marks = (start, end+1)
      if start not in urlreflist:
        try:
          marks['a'].append(tag_marks)
        except KeyError:
          marks['a'] = [tag_marks]
        ref = "url_http://%s?%s" % (server, quote_plus(item))
        try:
          marks[ref].append(tag_marks)
        except KeyError:
          marks[ref] = [tag_marks]
      if item and item[0] == ':':
        smiley = quote(item[1:])
        try:
          marks["smiley_"+smiley].append(tag_marks)
        except KeyError:
          marks["smiley_"+smiley] = [tag_marks]
    else:
      break
    start = str.find("[", end)
  
class PostParser(object):
  
  def __init__(self):
    self.clockParser = ClockParser()
    self.htmlParser = MiniHtmlParser()
  
  def parsePost(self, text, trib, wiki=None):
    """ Fonction pour parser, de manire gnrale, un post de la tribune.
        Il faut fournir le texte d'entre, la tribune de rfrence (pour
        le parsage des horloges et le reconnaissage des cross-rfrences),
        et l'adresse d'un ventuel serveur wiki si on veut.
        A la fin, on renvoie le texte transform (sans balises, etc.) et
        un dictionnaire tags => liste de coordonnes qui permettent de
        retrouver les styles  appliquer.
    """
    msg, tags = self.htmlParser.parse(text)
    clocks = self.clockParser.parse(msg, trib)
    for mark in clocks:
      tag_marks = (mark[0], mark[1])
      rtrib = mark[3]
      uref = "%s@%s" % (mark[2], rtrib)
      if rtrib == trib:
        tag_type = "clock"
      else:
        tag_type = "crossclock"
      try:
        tags[tag_type].append(tag_marks)
      except KeyError:
        tags[tag_type] = [tag_marks]
      try:
        tags[uref].append(tag_marks)
      except KeyError:
        tags[uref] = [tag_marks]
    if wiki:
      parseWikiUrls(msg, wiki, tags)
    return msg, tags
  
# Instances cres  l'avance pour speeder un peu
clockParser = ClockParser()
htmlParser  = MiniHtmlParser()
postParser  = PostParser()
