# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module plugin
##   Ce module contient les classes gnriques d'interface des plug-ins
## ----------------------------------------------------------------------

import sys, os, os.path
sys.path.append('..')

import logging
log = logging.getLogger('pycc')

from constants import *
from utils import *
from persistants import Style


class Plugin(object):
  
  def __init__(self, pycc, name, descr=""):
    self.pycc = pycc    # Pointeur vers pycc
    self.name = name    # Le nom tel qu'il apparat dans le menu
    self.descr = descr  # La description longue
    self.config_path = os.path.join(self.pycc.config_path, "plugins")
    self.style_name = 'plugin_%s' % self.name
    self.load_config()
    self.init()
    self.load_style()
  
  def __str__(self):
    return self.name
  
  def load_style(self):
    self.style = self.get_style()
    if self.style is not None:
      if not self.pycc.default_styles.exists(self.style_name):
        self.pycc.default_styles.add(self.style_name, self.style)
  
  def set_style_bouchot(self, bouchot):
    if self.style is not None:
      if not bouchot.styles.exists(self.style_name):
        bouchot.styles.add(self.style_name, self.pycc.default_styles.get(self.style_name))
        bouchot.styles.get(self.style_name).use_default = 1
        bouchot.apply_styles()
  
  def get_style(self):
    """ Renvoie un objet Style spcifique au plugin, s'il y a lieu """
    return None
  
  def load_config(self):
    """ Affecte  self.options l'ensemble des options de config du plug-in """
    self.options = None
  
  def init(self):
    """ Actions  raliser au moment de l'initialisation du plug-in """
    pass
  
  def config_frame(self, master):
    """ Construit la frame de configuration des options du plug-in """
    return None
  
  def process(self, bouchot, context, *args, **kwargs):
    """ Dfinit quoi faire lors de l'appel via une action """
    pass
  
  def action_menu(self):
    """ Dfinit quoi faire lors de l'appel via le menu """
    pass


class TextPlugin(Plugin):
  """ Classe spciale de plugins "transformeurs de texte"
  """
  def get_style(self):
    return None
    # return Style("[plugin] %s" % self.name, foreground='#ff0000')
  
  def action_menu(self):
    if self.pycc.palmi.palmi.selection_present():
      text = toUnicode(self.pycc.palmi.palmi.selection_get())
    else:
      ptxt = self.pycc.palmi.get()
      if len(ptxt) == 0: return
      text = ptxt[:-1]
    res = self.text_transform(text)
    self.pycc.palmi.palmi.inserter(res, del_selection=1)
  
  def text_transform(self, text, tags={}):
    return text
  
  def process(self, bouchot, context, post=None, *args, **kwargs):
    if context == PALMI:
      self.action_menu()
      return
    if post is None:
      return
    post.message = self.text_transform(post.message, post.tags)
    post.status.append(self.name)
    text, tags = bouchot.trib.formatPost(post)
    bouchot.trib.replaceFromTag("id%s" % post.id, text, tags)
    if self.style is not None:
      self.set_style_bouchot(bouchot)
      bouchot.trib.pinni.setStyle(post.id, self.style_name, 'message')
  

class AttribPlugin(Plugin):
  
  def __init__(self, pycc, name, descr=""):
    Plugin.__init__(self, pycc, name, descr)
    # attlist = self.pycc.options.val('available_attrs')
    if self.attrib not in available_attrs:
      #attlist[self.attpos:self.attpos] = [self.attrib]
      #self.pycc.options.set('available_attrs', attlist)
      available_attrs[self.attpos:self.attpos] = [self.attrib]
  
  def init(self):
    self.attrib = 'attribname'
    self.attpos = 0  # Attribute position
  
  def load_style(self):
    self.style_name = self.attrib
    Plugin.load_style(self)
  
  def action_menu(self):
    cur_trib = self.pycc.bouchots[self.pycc.get_bouchot()].trib
    if cur_trib is None or not cur_trib.loaded: return
    attlist = cur_trib.options.val('disp_post_attrs')
    if self.attrib in attlist:
      attlist.remove(self.attrib)
    else:
      attlist[self.attpos:self.attpos] = [self.attrib]
    cur_trib.options.set('disp_post_attrs', attlist)
    cur_trib.refresh_all()
  
  def compute_attrib(self, post):
    return "value"  # Abstract - calcul de la valeur du nouvel attribut de post
  
  def process(self, bouchot, context=PINNI, post=None, *args, **kwargs):
    if post is None: return
    post.__dict__[self.attrib] = self.compute_attrib(post)
    self.set_style_bouchot(bouchot)
