# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module palmi
##   Le Palmipde Editor.
##   (si vous trouvez que c'est le wingedge, jetez donc un oeil au pinni)
## ----------------------------------------------------------------------

from Tkinter import *
import Pmw, tkFont
from ScrolledText import ScrolledText
import string, re,  time

import logging
log = logging.getLogger('pycc')

from constants import *
from coinlib.utils import *
from coinlib.parsers import *
from coinlib.baseclasses import *
from coinlib.persistants import *
from cointklib.basicwidgets import *
from cointklib.widgets import *

import types
_unicode = types.UnicodeType

# Ensemble de raccourcis clavier dfinis par dfaut
default_bindings = {
    'Return':               PyccEvent(ActionPost()),
    'Control-Return':       PyccEvent(ActionPost(auth=0)),
    'Shift-Return':         PyccEvent(ActionPost(multi=1)),
    'Control-Shift-Return': PyccEvent(ActionPost(auth=0, multi=1)),
    'Alt-b': PyccEvent(ActionTagger("b")),
    'Alt-i': PyccEvent(ActionTagger("i")),
    'Alt-s': PyccEvent(ActionTagger("s")),
    'Alt-u': PyccEvent(ActionTagger("u")),
    'Alt-t': PyccEvent(ActionTagger("tt")),
    'Alt-m':     PyccEvent(ActionInsert("====><b> Moment %| </b><====")),
    'Control-w': PyccEvent(ActionInsert("%c")),
    'Alt-z':     PyccEvent(ActionInsert("La SuSE sa sent bon, sai libre")),
    'Alt-o':     PyccEvent(ActionInsert("_o/* <b>BLAM</b>!")),
    'Alt-p':     PyccEvent(ActionInsert("_o/* <b>paf</b>!")),
    'Alt-g':     PyccEvent(ActionInsert("Ta gueule pwet")),
    'Alt-c':     PyccEvent(ActionInsert("sale chauve")),
    'Alt-n':     PyccEvent(ActionInsert("ounet"))
  }

class PalmiKeySet(Writeable):
  """ Objet persistant ensemble des raccourcis clavier
  """
  def __init__(self, name="PalmiKeys", path="config", fileName=None):
    Writeable.__init__(self, name, path, fileName)
    if not self.load():
      for keystroke, keyevent in default_bindings.items():
        self.add(keystroke, keyevent)

class PalmiOptions(OptionSet):
  """ Options du palmipde Editor
  """
  def setName(self, name):
    self.name = "Palmi"
  
  def setObjects(self):
    self._objects = {
      'max_history': Option("Nombre de messages  conserver dans l'historique", 30, TYPE_INT),
      'history':     Option("Historique", [('', None, None, 0, 0)], TYPE_LIST),
      'toedit':      Option("Editables", { \
          "Palmipde Editor": [ 'max_history' ]
        } )
    }
  


class PalmiEditor(ScrolledText):
  """ L'diteur du Palmipde
  """
  def __init__(self, parent, palmi, **kw):
    log.debug("## Entering PalmiEditor.__init__")
    ScrolledText.__init__(self, parent, wrap=WORD, selectbackground=HITXT_BACK, **kw)
    self.root = parent   # Rfrence vers le widget parent
    self.palmi = palmi   # Rfrence vers le palmipde
    
    self._buftext = ''
    self.clocks = []
    
    self.bind("<KeyRelease>", self.keypressed)
    self.bind("<Enter>", lambda e, s=self: s.focus_set())
    self.bind("<Shift-Up>", self.palmi.history_up)
    self.bind("<Shift-Down>", self.palmi.history_down)
    self.bind("<Alt-Left>", self.palmi.prevTab)
    self.bind("<Alt-Right>", self.palmi.nextTab)
    self.bind("<Alt-Up>", self.palmi.focus_pinni)
    self.bind("<ButtonRelease-3>", self.ctxt_popup)
  
  def ctxt_popup(self, event=None):
    tags = self.tag_names("@%d,%d" % (event.x, event.y))
    for t in tags:
      if t[:9] == "urlbloub_":
        refs, read = self.palmi.pycc.getBloub(t[9:])
        text = u"URL dj poste"
        for ref in refs:
          time, bouchot = ref.split("@")
          text += u"\nsur %s" % bouchot
          if time:
            text += u" ( %s)" % time
        if read:
          text += u"\net en plus, tu l'as dj lue [:kiki]"
        SplashWindow(self.palmi.root, u"bloub", text)
  
  def keypressed(self, event=None):
    log.debug("## Entering PalmiEditor.keypressed(%s)" % event)
    if not self.palmi.showhist:
      self.palmi.postinfo.pack_forget()
    self.palmi.showhist = 0
    self.hsyntax()
  
  def inserter(self, text, pos=None, del_selection=0):
    log.debug("## Entering PalmiEditor.inserter(%s, %s, %s)" % (text, pos, del_selection))
    if pos is None:
      pos = self.index(INSERT)
    else:
      pos = self.index("0.0+%sc" % pos)
    if del_selection:
      if self.selection_present():
        # print "--- selection", len(self.tag_ranges(SEL)), self.tag_ranges(SEL)
        start, end = self.tag_ranges(SEL)
        self.delete(start, end)
        pos = start
      else:
        self.deleter()
        pos = END
    cur = text.find('%|')
    if cur > 0:
      text = text.replace('%|', '')
    self.insert(pos, text)
    if cur > 0:
      if self.selection_present():
        thesel = self.selection_get()
        start = self.index(ANCHOR)
        end = self.index("%s+%sc" % (start, len(thesel)))
        if self.get(start, end) == thesel:
          self.delete(start, end)
          pos = self.index("%s-%sc" % (pos, len(thesel)))
        else:
          self.delete(self.index("%s-%sc" % (start, len(thesel))), start)
        self.insert(self.index("%s+%sc" % (pos, cur)), thesel)
      self.mark_set(INSERT, self.index("%s+%sc" % (pos, cur)))
    self.hsyntax()
    self.focus_set()
  
  def deleter(self):
    log.debug("## Entering PalmiEditor.deleter")
    self.delete(0.0, END)
    self._buftext = ''
  
  def tagger(self, tag):
    log.debug("## Entering PalmiEditor.tagger(%s)" % tag)
    if self.selection_present():
      thesel = self.selection_get()
      start = self.index(ANCHOR)
      end = self.index("%s+%sc" % (start, len(thesel)))
      if self.get(start, end) == thesel:
        self.insert(ANCHOR, '<'+tag+'>')
        self.insert("%s+%sc" % (end, (2+len(tag))), '</'+tag+'>')
      else:
        self.insert("%s-%sc" % (start, len(thesel)), '<'+tag+'>')
        self.insert(ANCHOR, '</'+tag+'>')
      self.selection_range(INSERT, INSERT)
    else:
      self.insert(INSERT, '<'+tag+'></'+tag+'>')
      self.mark_set(INSERT, "%s-%sc" % (self.index(INSERT), (3+len(tag))))
    self.hsyntax()
    self.focus_set()
  
  def selection_clear(text):
    text.tag_remove(SEL, 1.0, END)
  
  def selection_from(text, index):
    text._anchor = index
  
  def selection_present(text):
    return len(text.tag_ranges(SEL)) != 0
  
  def selection_range(text, start, end):
    text.tag_remove(SEL, 1.0, start)
    text.tag_add(SEL, start, end)
    text.tag_remove(SEL, end, END)
  
  def selection_to(text, index):
    if text.compare(index, "<", text._anchor):
      selection_range(text, index, text._anchor)
    else:
      selection_range(text, text._anchor, index)
  
  def hsyntax(self, event=None):
    log.debug("## Entering PalmiEditor.hsyntax")
    if self.palmi.dummy_key:
      self.delete(self.index(INSERT+"-1c"))
      self.palmi.dummy_key = 0
    curtext = toUnicode(self.get(0.0, END), "utf-8")
    if curtext != self._buftext:
      self.parse(curtext)
      self._buftext = curtext
  
  def remove_all_tags(self):
    log.debug("## Entering PalmiEditor.remove_all_tags")
    for tag in self.palmi.styles.get().keys():
      if tag != 'base':
        self.tag_remove(tag, 0.0, 'end')
  
  def parse(self, inputstring):
    # AFR: Regexp pour matcher des urls cf ci-dessous
    """((ftp|https?):// ((\w|[-;?:@=&\$_.+!*'(~#%,])*) (\w|[-;/?:@=&\$_.+!*'(~#%,])*)"""
    log.debug("## Entering PalmiEditor.parse")
    text = toUnicode(inputstring, self.palmi.pycc.bouchots[self.palmi.cur_trib].trib.options.val('encoding'))
    self.remove_all_tags()
    self.bloubs = {}
    # self.clocks = []
    self.clocks = clockParser.parse(text, self.palmi.cur_trib)
    for marks in self.clocks:
      self.tag_add("horloge", "0.0+%dc" % marks[0], "0.0+%dc" % marks[1])
    url_types = ["http://", "ftp://", "https://", "ftps://"]
    idx = deb_balise = 0; curtag = bufurl = ur""
    for c in text:
      bufurl += c
      if curtag == "balise" and c.lower() not in ur"bitsu/>":
        self.tag_remove("balise", "0.0+%dc" % deb_balise, "0.0+%dc" % idx)
        curtag = ""
      if c == ur'<':
        curtag = "balise"
        deb_balise = idx
      elif c == ur'>':
        if curtag == "balise":
          self.tag_add(curtag, "0.0+%dc" % idx)
        curtag = ""
      elif bufurl in url_types:
        curtag = "url"
        self.tag_add(curtag, "0.0+%dc" % (idx-len(bufurl)+1), "0.0+%dc" % idx)
        bufurltype = bufurl
        bufurl = ur""
      elif c == '%':
        if curtag != "url":
          curtag = "spec"
      if c in string.whitespace+ur"<>[];":
        bufurl = ur""
        if curtag == "spec" or curtag == "url":
          curtag = ""
      if curtag:
        self.tag_add(curtag, "0.0+%dc" % idx)
        if curtag == "url":
          # print "####", bufurl
          url = bufurltype + bufurl
          if self.palmi.pycc.isUrlBloub(url):
            self.tag_add("urlbloub", "0.0+%dc" % (idx-len(url)+1), "0.0+%dc" % (idx+1))
            self.tag_add("urlbloub_%s" % url, "0.0+%dc" % (idx-len(url)+1), "0.0+%dc" % (idx+1))
            # curtag = "urlbloub"
      idx += 1
    # AVT ici parse clocks
    # print "---- cur_trib is:", self.palmi.cur_trib
    maxlen = self.palmi.pycc.bouchots[self.palmi.cur_trib].trib.options.val('post_max_length')
    if len(text) > maxlen:
      self.tag_add("depasse", "0.0+%dc" % maxlen, 'end+1c')
  
  def update(self, ntrib):
    log.debug("## Entering PalmiEditor.update(%s)" % ntrib)
    self.clocks.sort()
    self.clocks.reverse()
    for mark in self.clocks:
      pos1, pos2, ref, trib = mark
      if "url" in self.tag_names("0.0+%dc" % pos1):
        continue
      if trib == ntrib:
        self.delete("0.0+%dc" % (pos2-len(trib)-1), "0.0+%dc" % pos2)
      else:
        if trib == self.palmi.cur_trib:
          if "@" not in self.get("0.0+%dc" % pos1, "0.0+%dc" % pos2):
            self.insert("0.0+%dc" % pos2, "@"+trib)
  

class MsgViewer(ScrolledText):
  """ L'onglet d'aperu de message du Palmipde
  """
  def __init__(self, parent, palmi, **kw):
    log.debug("## Entering MsgViewer.__init__")
    self.root = parent
    self.palmi = palmi
    ScrolledText.__init__(self, parent, wrap=WORD, **kw)
  
  def parseUrls(self, msg):
    log.debug("## Entering msgViewer.parseUrls(%s)" % msg)
    text = msg
    urlprefs = [ 'http', 'ftp', 'https', 'ftps' ]
    for prefix in urlprefs:
      start = -1
      pos = text.find('%s://' % prefix)
      # print pos
      while pos > start:
        end = pos+6
        while end < len(text) and text[end] not in string.whitespace: end += 1
        text = text[:pos] + '<a href="' + text[pos:end] + '"><b>[url]</b></a>' + text[end:]
        start = end
        pos = text.find('%s://' % prefix, start)
    return text
  
  def update(self, text, ident, cur_trib):
    log.debug("## Entering MsgViewer.update(%s, %s, %s)" % (text, ident, cur_trib))
    self.config(state=NORMAL)
    inputmsg = self.parseUrls(text)
    now = time.localtime()
    htime = time.strftime("%H:%M:%S", now)
    myhash = { '%l': ident.login, '%u': ident.ua, '%h': htime, '%v': PYCC_VERSION }
    inputmsg = escape_spec_chars(inputmsg, myhash)
    msg, tags = postParser.parsePost(inputmsg, cur_trib) # , old_slip=1)
    if cur_trib in ident.cookies.keys():
      pseudo = ident.login
    else:
      pseudo = ''
    ctime = time.strftime("%Y%m%d%H%M%S", now)
    npost = Post('42', ctime, pseudo, ident.ua, msg, tags)
    output_text, output_tags = self.palmi.pycc.bouchots[cur_trib].trib.formatPost(npost)
    self.delete(0.0, END)
    if type(output_text) == _unicode:
      self.insert(0.0, output_text)
    else:
      try:
        self.insert(0.0, output_text.decode(self.palmi.pycc.bouchots[cur_trib].trib.options.val('encoding')))
      except:
        print "### update palmi ratai"
        self.insert(0.0, output_text)
    for tag in output_tags.keys():
      for range in output_tags[tag]:
        self.tag_add(tag, "0.0+%sc" % range[0], "0.0+%sc" % range[1])
    self.config(state=DISABLED)
    apply_styles(self, self.palmi.pycc.bouchots[cur_trib].styles, 'pinni', self.palmi.pycc.default_styles)
  

class SearchFrame(Frame):
  """ L'onglet "Recherche" du Palmipde
  """
  def __init__(self, parent, palmi, **kwargs):
    log.debug("## Entering SearchFrame.__init__")
    self.root = parent
    self.palmi = palmi
    Frame.__init__(self, parent, **kwargs)
    self.lastpos = 1.0
    self.search_buf = ""
    self.buf_trib = None
    self.body()
  
  def body(self):
    log.debug("## Entering SearchFrame.body")
    self.find = TextField(self.root, "Chercher :")
    self.find.pack(anchor=W)
    self.find.field.bind("<KeyRelease>", self.keypressed)
    fr = Frame(self.root)
    self.case = BoolField(fr, "Case sensitive", 0)
    self.case.pack(side=LEFT)
    self.regexp = BoolField(fr, "Expression rgulire", 0)
    self.regexp.pack(side=RIGHT)
    fr.pack(anchor=W)
    self.alltabs = BoolField(self.root, "Rechercher dans tous les onglets", 1)
    self.alltabs.pack(anchor=W)
    self.find.focus_set()
  
  def keypressed(self, event):
    log.debug("## Entering SearchFrame.keypressed")
    self.search_buf = self.find.get()
    k = str(event.keysym)
    if k in ignorekeys+modifkeys:
      return
    if k != "Return":
      if self.regexp.get(): return
      self.lastpos = 1.0
    self.findnext()
  
  def clearsearch(self, trib):
    self.lastpos = 1.0
    self.buf_trib = None
    if trib is not None:
      trib.pinni._pinni.tag_remove('hilifound', 1.0, END)
    self.palmi.pycc.setStatus("")
  
  def findnext(self, event=None):
    log.debug("## Entering SearchFrame.findnext")
    cur_trib = self.palmi.pycc.getCurTrib()
    if not self.search_buf:
      self.clearsearch(cur_trib)
      return
    if cur_trib is not None:
      self.lastpos = cur_trib.pinni.search(self.search_buf, self.lastpos, self.case.get(), self.regexp.get(), END)
    if self.lastpos is None or cur_trib is None:
      # print "---- lastpos found=", self.lastpos
      self.lastpos = 1.0
      if self.alltabs.get():
        if self.buf_trib is None:
          self.buf_trib = cur_trib.bouchot.name
        self.palmi.pycc.nextTab()
        self.find.focus_set()
        if self.buf_trib == self.palmi.pycc.get_bouchot():
          self.buf_trib = None
          return
        self.findnext()
    else:
      self.lastpos = cur_trib.pinni._pinni.index("%s+%dc" % (self.lastpos, len(self.search_buf)))
  

class Palmipede(Frame):
  """ Le Palmipde itself
  """
  def __init__(self, parent, pycc, **kwargs):
    log.debug("## Entering Palmipede.__init__")
    self.root = parent
    self.pycc = pycc
    self.init()
    Frame.__init__(self, parent, **kwargs)
    self.build_interface()
  
  def init(self):
    log.debug("## Entering Palmipede.init")
    self.cur_trib = None
    self.cur_id = None
    self.id = None
    self.tablist = ['Message', 'Aperu', 'Identit', 'Recherche']
    self.showhist = 0
    self.dummy_key = 0
    self.history_cpt = 0
    self.load_config()
    self.apply_config()
    self.history = self.options.val('history')
    # self.bloubs = {}
  
  def kill(self):
    log.debug("## Entering Palmipde.kill")
    self.options.set('history', self.history)
    self.save_config()
    self.destroy()
  
  # ------------------------------------------
  # Mthodes pour grer la config
  # ------------------------------------------
  
  def load_config(self):
    self.options = PalmiOptions(path=self.pycc.config_path)
    self.styles = PalmiStyleSet(path=self.pycc.config_path)
    self.keys = PalmiKeySet(path=self.pycc.config_path)
  
  def apply_config(self):
    pass # Rien de spcial  faire
  
  def save_config(self):
    log.debug("## Entering Palmipede.save_config")
    self.options.save()
    self.keys.save()
    self.styles.save()
  
  def apply_styles(self):
    apply_styles(self.palmi, self.styles, 'palmi')
  
  # ------------------------------------------
  # Mthodes pour l'interface graphique
  # ------------------------------------------
  
  def build_interface(self):
    log.debug("## Entering Palmipede.body")
    self.tabs = Pmw.NoteBook(self, raisecommand=self.tabselect, tabpos='n', borderwidth=1)
    self.tabs.pack(side="left", fill="both", expand=1, padx=0, pady=0)
    for tab in self.tablist:
      page = self.tabs.add(tab)
      if tab == "Message":
        self.palmi = PalmiEditor(page, self, width=40, height=4)
        self.palmi.pack(fill=BOTH, expand=1)
        self.apply_styles()
        self.tb = self.toolbar(page)
        self.tb.pack(side="bottom", anchor=W)
      elif tab == "Aperu":
        self.pview = MsgViewer(page, self, background="#dfffe0", width=40, height=6)
        self.pview.pack(fill=BOTH, expand=1)
      elif tab == "Identit":
        scfr = Pmw.ScrolledFrame(page, usehullsize=1)
        scfr.interior().config(height=10, width=10)
        self.id = CoinObjectEditor(scfr.interior(), None)
        scfr.pack(fill=BOTH, expand=1)
        # self.id.pack()
      else: # tab == "Recherche"
        scfr = SearchFrame(page, self)
        scfr.pack(fill=BOTH, expand=1)
    self.tabs.setnaturalsize()
    fr = Frame(self.root)
    self.sideinfo = Label(fr, text="")
    self.sideinfo.pack(side="top", pady=5)
    self.tribsel = ComboField(fr, "Tribune:", self.pycc.bouchots.keys())
    self.tribsel.pack(side="top", anchor=NE)
    self.tribsel.bind_onchange(self.change_trib)
    self.idsel = ComboField(fr, "Identit:", self.pycc.identities.get().keys())
    self.idsel.bind_onchange(self.change_id)
    self.idsel.pack(side="top", anchor=NE)
    self.postinfo = Label(fr, text="", font="Helvetica 8", foreground="#600060", relief=GROOVE, border=2)
    fr.pack(side="right", fill=Y)
  
  def toolbar(self, parent):
    log.debug("## Entering Palmipede.toolbar")
    tb = ToolBar(parent)
    tb.addButton('b', text="B", command=lambda p=self.palmi: p.tagger("b"))
    tb.buttons['b'].config(font=tkFont.Font(family="Times", size=9, weight=tkFont.BOLD))
    tb.addButton('i', text="I", command=lambda p=self.palmi: p.tagger("i"))
    tb.buttons['i'].config(font=tkFont.Font(family="Times", size=9, weight=tkFont.BOLD, slant=tkFont.ITALIC))
    tb.addButton('u', text="U", command=lambda p=self.palmi: p.tagger("u"))
    tb.buttons['u'].config(font=tkFont.Font(family="Times", size=9, underline=1))
    tb.addButton('s', text="S", command=lambda p=self.palmi: p.tagger("s"))
    tb.buttons['s'].config(font=tkFont.Font(family="Helvetica", size=8, overstrike=1))
    tb.addButton('tt', text="TT", command=lambda p=self.palmi: p.tagger("tt"))
    tb.buttons['tt'].config(font=tkFont.Font(family="Courier", size=8))
    tb.addButton('m', text="=>M<=", command=lambda p=self.palmi: p.inserter("====><b> Moment %| </b><===="))
    tb.buttons['m'].config(width=9)
    tb.addButton('l', text="login", command=lambda p=self.palmi: p.inserter("%l"))
    tb.buttons['l'].config(width=5)
    tb.addButton('u', text="UA", command=lambda p=self.palmi: p.inserter("%u"))
    tb.buttons['u'].config(width=5)
    return tb
  
  def bindKeys(self):
    log.debug("## Entering Palmipede.bindKeys")
    for keystroke, keyevent in self.keys.get().items():
      try:
        self.palmi.bind("<%s>" % keystroke, lambda e, s=self, ks=keystroke, ke=keyevent: s.keyevent(ks, ke))
      except:
        log.warning("Palmipede: Impossible de configurer le raccourci clavier %s" % keystroke)
  
  def unbindKeys(self):
    log.debug("## Entering Palmipede.unbindKeys")
    for keystroke in self.keys.get().keys():
      self.palmi.unbind("<%s>" % keystroke)
  
  def keyevent(self, ks, ke):
    log.debug("## Entering Palmipede.keyevent(%s)" % ks)
    if "Return" in ks: self.dummy_key = 1
    ke.action.process(self, context=PALMI)
  
  def focus_pinni(self, event=None):
    trib = self.pycc.getCurTrib()
    if trib is not None:
      trib.pinni.focus_set()
  
  def change_tab(self, event=None):
    list = self.tabs.pagenames()
    idx = list.index(self.tabs.getcurselection())
    self.dummy_key = 1
    self.tabselect(list[(idx+1)%len(list)])
  
  def nextTab(self, event=None):
    self.pycc.nextTab()
    self.palmi.focus_set()
    self.palmi.mark_set(INSERT, self.palmi.index("%s-1c" % INSERT))
  
  def prevTab(self, event=None):
    self.pycc.prevTab()
    self.palmi.focus_set()
    self.palmi.mark_set(INSERT, self.palmi.index("%s+1c" % INSERT))
  
  # ------------------------------------------
  # Mthodes spcifiques
  # ------------------------------------------
  
  def update_id_list(self):
    log.debug("## Entering Palmipede.update_id_list")
    prev_id = self.idsel.get()
    newlist = self.pycc.identities.get().keys()
    self.idsel.setitems(newlist)
    if prev_id in newlist:
      self.set_id(prev_id)
  
  def update_trib_list(self):
    log.debug("## Entering Palmipede.update_trib_list")
    newlist = self.pycc.activeBouchots()
    self.tribsel.setitems(newlist)
  
  def change_id(self):
    log.debug("## Entering Palmipede.change_id")
    id = self.idsel.get()
    if id == self.cur_id: return
    if not id: return
    self.set_id(id)
  
  def change_trib(self, event=None):
    log.debug("## Entering Palmipede.change_trib")
    trib = self.tribsel.get()
    if trib == self.cur_trib: return
    if not trib: return
    self.set_trib(trib)
  
  def set_id(self, id):
    log.debug("## Entering Palmipede.set_id(%s)" % id)
    self.idsel.set(id)
    parent = self.id.parent
    self.id.destroy()
    self.id = IdentityEditor(parent, self.get_id())
    tmplist = {}
    for name, bouchal in self.pycc.bouchots.items():
      tmplist[name] = bouchal.trib
    self.id.set_trib_list(tmplist)
    self.id.pack()
    self.cur_id = id
    self.update_apercu()
  
  def set_trib(self, trib):
    log.debug("## Entering Palmipede.set_trib(%s)" % trib)
    if trib not in self.pycc.activeBouchots():
      return
    if self.pycc.bouchots[trib].trib is None:
      return
    if trib != self.cur_trib:
      self.tribsel.set(trib)
      self.palmi.update(trib)
      self.cur_trib = trib
      self.palmi.hsyntax()
    id = self.pycc.bouchots[trib].options.get('default_id').get()
    self.set_id(id)
    if trib != self.pycc.get_bouchot():
      eval("self.palmi.config(%s)" % self.styles.get('hilight').get())
    else:
      eval("self.palmi.config(%s)" % self.styles.get('base').get())
  
  def get_id(self):
    log.debug("## Entering Palmipede.get_id")
    id = self.idsel.get()
    if not id:
      log.warning("Palmipede.get_id: aucune identit disponible")
      return None
    return self.pycc.identities.get(id)
  
  def get_trib(self):
    log.debug("## Entering Palmipede.get_trib")
    if not self.cur_trib:
      log.warning("Palmipede.get_trib: aucune tribune courante slectione")
      return None
    return self.cur_trib
  
  def get(self):
    log.debug("## Entering Palmipede.get")
    return toUnicode(self.palmi.get(0.0, END), "utf-8")
  
  def update_apercu(self):
    log.debug("## Entering Palmipede.update_apercu")
    cur_id = self.get_id()
    if cur_id is None: return
    cur_trib = self.get_trib()
    if cur_trib is None: return
    self.pview.update(self.get(), cur_id, cur_trib)
  
  def tabselect(self, tab):
    log.debug("## Entering Palmipede.tabselect(%s)" % tab)
    if self.tabs.getcurselection() != 'Identit' and self.id is not None:
      self.id.apply()
    if tab == 'Aperu':
      self.update_apercu()
    elif tab == 'Identit':
      self.change_id()
    self.tabs.selectpage(tab)
  
  def mouler(self, auth=1, multi=0):
    log.debug("## Entering Palmipede.mouler(%s, %s)" % (auth, multi))
    cur_trib = self.get_trib()
    cur_id = self.get_id()
    destlist = [cur_trib]
    if multi:
      destlist += [ t for t, b in self.pycc.bouchots.items() if b.trib is not None and b.trib.loaded and t != cur_trib]
    for trib in destlist:
      if trib == cur_trib:
        id = cur_id
        addtohist = 1
      else:
        id = None
        addtohist = 0
        self.palmi.update(trib)
      msg = self.get()[:-1]
      log.info("Post dans %s: '%s'" % (trib, msg))
      self.pycc.bouchots[trib].trib.poster(msg, auth, id)
      if addtohist:
        self.history_add(msg, trib, id, auth, multi)
    self.palmi.deleter()
  
  # ------------------------------------------
  # Mthodes de gestion de l'historique
  # ------------------------------------------
  
  def history_up(self, event=None):
    log.debug("## Entering Palmipede.history_up")
    self.history_go(1)
  
  def history_down(self, event=None):
    log.debug("## Entering Palmipede.history_down")
    self.history_go(-1)
  
  def history_go(self, offset):
    log.debug("## Entering Palmipede.history_go(%s)" % offset)
    self.history_cpt = (self.history_cpt + offset) % len(self.history)
    self.history_show()
  
  def history_show(self):
    log.debug("## Entering Palmipede.Show_History(%s)" % self.history_cpt)
    self.showhist = 1
    msg, trib, id, auth, multi = self.history[-self.history_cpt]
    if trib: self.set_trib(trib)
    if id: self.set_id(id)
    self.palmi.deleter()
    self.palmi.inserter(msg)
    if msg.strip() != "":
      info = ""
      if not auth:
        info += " Anonyme "
      if multi:
        info += " Multi "
      if info != "":
        self.postinfo.config(text=info)
        self.postinfo.pack(side="bottom", fill=X, pady=4)
      else:
        self.postinfo.pack_forget()
  
  def history_add(self, msg, trib=None, id=None, auth=1, multi=0):
    log.debug("## Entering Pamipede.history_add(%s, %s, %s, %s, %s)" % (msg, trib, id, auth, multi))
    if trib is None:
      trib = self.get_trib()
    if id is None:
      id = self.get_id()
    self.history.append((msg, trib, id.login, auth, multi))
    self.history_cpt = 0
    if len(self.history) > self.options.val('max_history'):
      self.history.pop(1)
  
