# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module pikaplop
##   Ce module dfinit les plug-ins Plopifier et Pikachizer
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')
import sys
sys.path.append('..')
from coinlib.plugin import *
from coinlib.persistants import Option, OptionSet
from cointklib.widgets import OptionsEditor
from cointklib.basicwidgets import *

import string, re, urllib
from random import randrange, choice

def build_plugins(pycc):
  return { \
    'plopifier': Plopifier(pycc, "Plopifier", "Traduit le texte en plop meuh de gruiiik limule ka-pika"),
    'pikachize': Pikachizer(pycc, "Pikachizer", "Traduit le texte en langage Pikachu")
  }

class PlopOptions(OptionSet):
  def setName(self, name):
    self.name = "Plopifier"
  
  def setObjects(self):
    self._objects = {
      'plop_words': Option("Liste des mots-plop", value=[ 
          'pl0p', 'co1n', 'pika', 'plip', 'zzZz', 'burp',
          'meuuh', 'kikoo', 'pouet', 'prout', 'gaaaa', 'moule', 'miaou',
          'plooop', 'gnegne', 'gargle', 'blabla', 'limule', 'gloups',
          'nartaaa', 'gruiiik', 'blouark', 'flurk', 'ka-pika',
          'glouglou', 'kapou', 'buarrgll', 'coincoin',
          'fmelebele', 'mollusque', 'coccimule', 'pschiittt',
          'croucroute', 'piiikachuu', 'bzzonnng',
          'rastakoure', 'wolfenstein', 'porteninwak',
          'britneycdkey', 'super-tomate', 'eviv-bulgroz',
          'obiwan-kenobi', 'bananemagique'
        ], type=TYPE_LIST),
      'plop_density': Option("Densit de la plopification (en %)", value=75, type=TYPE_INT),
      'plop_exclude': Option("Expressions  ne pas plopifier", value="^.*([0-9=:]+|\[url\]).*$"),
      'toedit':    Option("Editables", { \
          "Gnral": [ 'plop_words', 'plop_density', 'plop_exclude' ]
        } )
    }

class PlopifierEditor(OptionsEditor):
  
  def body(self):
    OptionsEditor.body(self)
    fr = Frame(self.tab)
    self.plopword = TextField(fr, "Nouveau mot-plop:")
    self.plopword.pack(side=LEFT)
    Button(fr, text="Ajouter", command=self.new).pack(side=LEFT)
    fr.pack(anchor=W)
  
  def disp_list(self, fr, name, option):
    Label(fr, text=option.descr).pack(pady=2)
    self.widgets[name] = SortedListBox(fr, option.value, width=15, height=10)
    self.widgets[name].pack(fill=X)
    # self.widgets[name].bind_new(self.new)
  
  def new(self):
    plopword = self.plopword.get()
    if not plopword:
      return
    self.widgets['plop_words'].add(plopword)
  
  def apply(self):
    for opt, widget in self.widgets.items():
      if opt == 'plop_words':
        self.object.set(opt, widget.getAll())
      else:
        self.object.set(opt, widget.get())
    self.object.save()

class Plopifier(TextPlugin):
  
  def init(self):
    self.plop_exclude = re.compile(self.options.val('plop_exclude'))
  
  def load_config(self):
    self.options = PlopOptions(path=self.config_path)
  
  def config_frame(self, master):
    return PlopifierEditor(master, self.options, self.pycc)
  
  def action_menu(self):
    self.pycc.showConfig('cplug_plopifier')
  
  def get_style(self):
    return Style("[plugin] %s" % self.name, font=('Verdana', 8, 'italic'), foreground='#a0a0a0')
  
  def formap(self, item):
    res = item
    if not self.plop_exclude.match(item):
      if randrange(100) < self.options.val('plop_density'):
        list = [w for w in self.options.val('plop_words') if len(w) == len(item)]
        if len(list) > 0:
          res = choice(list)
    return res
  
  def text_transform(self, text, tags={}):
    return " ".join(map(self.formap, text.split()))
  

class Pikachizer(TextPlugin):
  
  def get_style(self):
    return Style("[plugin] %s" % self.name, foreground='#f0f000')
  
  def text_transform(self, text, tags={}):
    try:
      query = urllib.urlencode({'text': text})
      f = open_url("pikachize.eye-of-newt.com", "/pika.cgi?%s" % query, self.pycc.options)
      res = f.read()
      f.close()
      start = res.find("<PRE>") +5
      end = res.find("</PRE>", start)
      return res[start:end]
    except:
      self.error = "Pikachization impossible (erreur rseau ou du serveur pikachize)"
  
