# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Plug-in UpdateConfig
##   Plug-in pour gnrer les fichiers de configuration automatique
##    partir de la configuration actuelle de pycc
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')
import sys
sys.path.append('..')

from coinlib.plugin import *
from coinlib.persistants import Writeable, Option, OptionSet
from cointklib.basicwidgets import *
from cointklib.widgets import OptionsEditor, CoinObjectEditor
from cointklib.dialogs import OptionsDialog

import os, os.path

def build_plugins(pycc):
  return { \
    'updconf': UpdateAutoConfig(pycc, "Update autoconfig", "Gnre les fichiers pour la config auto")
  }

class UpdConfOptions(OptionSet):
  
  def setName(self, name):
    self.name = "UpdConf"
  
  def setObjects(self):
    self._objects = {  
      'dest_rep':  Option("Rpertoire de stockage des .xml", "", TYPE_STR),
      'gen_inc':   Option("Gnrer bouchots.inc", 0, TYPE_BOOL),
      'desc_list': Option("Descriptions des bouchots", {}, 'desc_hash'),
      'toedit':    Option("Editables", {"Gnral": ['dest_rep', 'gen_inc', 'desc_list']})
    }
  
class OptionSetClone(Writeable):
  
  def __init__(self, name, path, optionset, excludeset=[]):
    self.optionset = optionset
    self.name = name
    Writeable.__init__(self, name, path, None, None)
    self._objects = {}
    for option, value in self.optionset.get().items():
      if option not in excludeset:
        self.add(option, value)
    try:
      toedit = self.optionset.val('toedit')
    except:
      toedit = {"Gnral": []}
    for section, optlist in toedit.items():
      for option in optlist:
        if option not in self.get().keys():
          continue
        self.get(option).writeable_attrs.append('toedit')
        self.get(option).toedit = section
    self.save()
  
class DescEditor(CoinObjectEditor):
  
  def body(self):
    self.w_descr = TextField(self._fr, "Description", self.object[0])
    self.w_descr.pack(anchor='e')
    value = self.object[1]
    if not value:
      value = "%s %s %s" % (self.extra_arg.options.val('http_server'), self.extra_arg.name, self.extra_arg.options.val('bouchot_aliases'))
    self.w_keywords = TextField(self._fr, "Mots-cls", value)
    self.w_keywords.pack(anchor='e')
    self.w_public = BoolField(self._fr, "Apparait sur la liste publique", self.object[2])
    self.w_public.pack(anchor='w')
    """### TEMPORAIRE
    bouchot = self.extra_arg.name
    #f = open("/home/chrisix/.pycc08/plugins/bouchots.inc")
    f = open("D:\\tools\\Users\\pycc-0.8.6\\plugins\\bouchots.inc")
    line = "coucou"
    while line:
      line = f.readline()
      print "[", line
      if line.find("'%s' => array" % bouchot) >= 0:
        desc = f.readline().strip()[1:-2]
        url = f.readline()
        keywords = f.readline()
        public = (f.readline().find("true") >= 0)
        self.w_descr.set(desc)
        self.w_public.set(public)
        break
    f.close()
    ### FIN TEMPORAIRE"""
  
  def get(self):
    descr = self.w_descr.get()
    try:
      descr = descr.encode('iso-8859-15')
    except:
      pass
    keywords = self.w_keywords.get()
    try:
      keywords = keywords.encode('iso-8859-15')
    except:
      pass
    public = self.w_public.get()
    return (descr, keywords, public)
  
class DescHashEditor(CoinObjectEditor):
  
  def body(self):
    blist = self.extra_arg.pycc.bouchots.keys()
    Label(self._fr, text="Liste descriptive des bouchots", background=COLOR_TITLE).pack(fill=X, anchor=NW)
    self.fr = Frame(self._fr)
    self.w_blist = SortedListBox(self.fr, blist, height=8, width=20)
    self.w_blist.pack(side=LEFT, fill=Y, expand=1, anchor=NW)
    self.w_blist.bind_select(self.select)
    self.w_desc = {}
    for bouchot in blist:
      try:
        tupdesc = self.object.get()[bouchot]
      except:
        tupdesc = ("Bouchot %s" % bouchot, "", 1)
      self.w_desc[bouchot] = DescEditor(self.fr, tupdesc, self.extra_arg.pycc.bouchots[bouchot])
    self.fr.pack(anchor=NW)
  
  def select(self):
    for bouchot in self.w_desc.keys():
      self.w_desc[bouchot].pack_forget()
    cur_bouchot = self.w_blist.get()
    if not cur_bouchot: return
    self.w_desc[cur_bouchot].pack(side=LEFT, padx=5, pady=5, anchor=NW)
  
  def apply(self):
    self.object.set(self.get())
  
  def get(self):
    tmp_hash = {}
    for bouchot in self.w_desc.keys():
      tmp_hash[bouchot] = self.w_desc[bouchot].get()
    return tmp_hash
    

class UpdConfEditor(OptionsEditor):
  
  def disp_other(self, fr, name, option):
    if name == 'desc_list':
      self.widgets[name] = DescHashEditor(fr, option, self.extra_arg)
      self.widgets[name].pack(pady=2, fill=X)
    else:
      self.widgets[name] = TextField(fr, option.descr, value=option.get())
      self.widgets[name].pack(pady=2, fill=X)


class UpdConfDialog(OptionsDialog):
  
  def body(self, master):
    self.w_options = UpdConfEditor(master, self.object, extra_arg=self.extra_arg)
    self.w_options.pack()
  

class UpdateAutoConfig(Plugin):
  
  def init(self):
    self.EXCLUDE = {}
    self.EXCLUDE['bouchot']   = ['toedit', 'startup_modules', 'default_id']
    self.EXCLUDE['tribune']   = ['toedit', 'docked', 'disp_post_attrs']
    self.EXCLUDE['rssreader'] = ['toedit', 'docked', 'read_at_exp']
    self.EXCLUDE['forums']    = ['toedit', 'docked', 'read_at_exp']
    self.EXCLUDE['multirss']  = ['toedit', 'docked', 'read_at_exp']
    self.EXCLUDE['userinfo']  = ['toedit', 'docked']
  
  def load_config(self):
    self.options = UpdConfOptions(path=self.config_path)
    if not self.options.val('dest_rep'):
      self.options.set('dest_rep', os.path.join(self.pycc.config_path, "autoconfig"))
  
  def config_frame(self, master):
    return UpdConfEditor(master, self.options, self)
  
  def action_menu(self):
    self.pycc.root.canceled = 1
    UpdConfDialog("Update Autoconfig", self.pycc.root, self.options, self)
    # self.options.save()
    # print self.options
    if self.pycc.root.canceled:
      return
    self.gen_config()
    if self.options.val('gen_inc'):
      self.gen_incfile()
  
  def gen_incfile(self):
    descr = self.options.val('desc_list')
    header = "<?php\n\n  $bouchots = array(\n"
    footer = "\n  );\n?>"
    bodylist = []
    for bname, bouchot in self.pycc.bouchots.items():
      item =  "    '%s' => array(\n" % bname
      item += "      \"%s\",\n" % descr[bname][0]
      item += "      \"%s\",\n" % bouchot.options.val('http_server')
      item += "      \" %s\",\n" % descr[bname][1]
      if descr[bname][2]:
        item += "      true)"
      else:
        item += "      false)"
      bodylist.append(item)
    body = ",\n".join(bodylist)
    print "##", type(body)
    dfile = os.path.join(self.options.val('dest_rep'), "bouchots.inc")
    f = open(dfile, "w")
    f.write(header + body + footer)
    f.close()
  
  def gen_config(self):
    path = self.options.val('dest_rep')
    if not os.path.exists(path):
      try:
        os.mkdir(path)
      except:
        log.error("Plugin %s: Impossible de crer le rpertoire destination '%s'" % (self.name, path))
        return 0
    for bouchot in self.pycc.bouchots.values():
      self.gen_config_bouchot(bouchot)
  
  def gen_config_bouchot(self, bouchot):
    log.info("Update Autoconfig: write bouchot %s.xml" % bouchot.name)
    path = self.options.val('dest_rep')
    OptionSetClone(bouchot.name, path, bouchot.options, self.EXCLUDE['bouchot'])
    for modname, modtype in bouchot.options.val('base_modules').items():
      log.info("Update Autoconfig: write %s_%s.xml (module %s)" % (bouchot.name, modname, modtype))
      OptionSetClone("%s_%s" % (bouchot.name, modname), path, bouchot.modules[modname].options, self.EXCLUDE[modtype])
    