# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module pyc4.wizards
##   Ce module contient les assistants de configuration de pycc
## ----------------------------------------------------------------------

import sys
sys.path.append('..')
from urllib2 import HTTPError

from Tkinter import *
import tkMessageBox, tkSimpleDialog

from constants import *
from cointklib.dialogs import CoinTopLevel, OptionsDialog, LoginDialog
from cointklib.basicwidgets import *
from coinlib.persistants import *
from coinlib.utils import *

import logging
log = logging.getLogger('pycc')


class WizardStep(Frame):
  
  def __init__(self, wiz, islast=0, title=''):
    self.wiz = wiz        # Le wizard rattach
    self.title = title    # Titre de l'tape
    self.info = ''        # Texte pour la zone d'infos
    self.tags = {}        # Tags pour styles spciaux sur la zone d'infos
    self.next = None      # Pointeur sur le step suivant
    self.islast = islast  # True si ce step est la dernire tape du wizard
    Frame.__init__(self, self.wiz.fr_step)
    self.focus = self.body()
  
  def body(self):
    """ Classe abstraite, dfinit les widgets de l'tape du wizard
        Renvoie le widget qui doit recevoir le focus quand l'tape est charge
    """
    return None  # Override
  
  def getNext(self):
    if not self.validate():
      return None
    return self.next
  
  def validate(self):
    """ Classe abstraite, vrifications  effectuer pour valider l'tape
    """
    return 1     # Override
  

class Wizard(object):
  
  def __init__(self, root, title, image=None, isroot=0):
    if isroot:
      self.root = root
    else:
      self.root = Toplevel(root)
      self.root.transient(root)
      self.root.grab_set()
      self.root.geometry("+%d+%d" % (root.winfo_rootx()+50, root.winfo_rooty()+50))
    self.root.protocol("WM_DELETE_WINDOW", self.cancel)
    self.root.bind("<Escape>", self.cancel)
    self.title = title
    self.root.title(title)
    self.image = PhotoImage(file=image)
    self.build_interface()
  
  def build_interface(self):
    #== Image de gauche
    Label(self.root, image=self.image).pack(side=LEFT)
    
    #== Bannire de titre
    top_banner = Frame(self.root)
    #- Titre
    t = Label(top_banner, text=self.title+"  ", font="Times 12 bold italic", background=COLOR_TITLE)
    t.pack(side=LEFT)
    #- Texte "Etape x/n"
    self.steplabel = Label(top_banner)
    self.steplabel.pack(side=RIGHT)
    top_banner.pack(fill=X)
    
    #== Frame contenu principal
    fr = Frame(self.root, border=2, relief=GROOVE)
    #- Titre de l'tape
    self.step_title = Label(fr, font="Helvetica 10 bold")
    self.step_title.pack(padx=10)
    #- Blabla
    self.infos = Text(fr, wrap=WORD, cursor="arrow", background="#D0F0FF", relief=FLAT,
      bd=2, font="Helvetica 9", state=DISABLED, width=50, height=8)
    self.infos.tag_config('bold', font="Helvetica 9 bold")
    self.infos.tag_config('red', foreground="#FF4040")
    self.infos.tag_config('blue', foreground="#4040FF")
    self.infos.tag_config('hilight', background="#FFFCDA")
    self.infos.pack(padx=10, pady=10)
    #- Widgets
    self.steps = []   # Liste des steps effectus (objets de type WizardStep)
    self.fr_step = Frame(fr)
    self.fr_step.pack(pady=5, fill=BOTH)
    fr.pack(fill=BOTH, expand=1)
    
    #== Boutons 
    b = Frame(self.root)
    self.but_prev   = Button(b, text="< Prcdent", width=10, command=self.prev)
    self.but_next   = Button(b, text="Suivant >", width=10, command=self.next)
    self.but_end    = Button(b, text="Terminer", width=10, command=self.end)
    self.but_cancel = Button(b, text="Annuler", width=10, command=self.cancel)
    self.but_cancel.pack(side=RIGHT, padx=5)
    b.pack(pady=5, fill=X)
  
  def initialize(self):
    if not self.steps:
      self.steps.append(WizardStep(self, 1))
    self.showCurStep()
  
  def showCurStep(self):
    self.steps[-1].pack(padx=5, pady=5, fill=BOTH, expand=1)
    self.setButtons(self.steps[-1].islast)
    self.setProperties()
  
  def getNextStep(self):
    return self.steps[-1].getNext()
  
  def setInfo(self, text, tags={}):
    self.infos.config(state=NORMAL)
    self.infos.delete(0.0, END)
    self.infos.insert(END, text)
    for tag, ranges in tags.items():
      for range in ranges:
        self.infos.tag_add(tag, "0.0+%sc" % range[0], "0.0+%sc" % range[1])
    self.infos.config(state=DISABLED)
  
  def setStepTitle(self, title):
    self.step_title.config(text=title)
  
  def setProperties(self):
    self.steplabel.config(text="Etape %s" % len(self.steps))
    step = self.steps[-1]
    self.setStepTitle(step.title)
    self.setInfo(step.info, step.tags)
    self.fr_step.update_idletasks()
    if step.focus:
      step.focus.focus_set()
  
  def setButtons(self, islast=0):
    self.but_prev.pack_forget()
    self.but_next.pack_forget()
    self.but_end.pack_forget()
    if islast:
      self.but_end.pack(side=RIGHT)
      self.root.bind("<Return>", self.end)
      self.but_cancel.config(state=DISABLED)
    else:
      self.but_next.pack(side=RIGHT)
      self.root.bind("<Return>", self.next)
      self.but_cancel.config(state=NORMAL)
    if len(self.steps) > 1:
      self.but_prev.pack(side=RIGHT)
  
  def prev(self, event=None):
    step = self.steps.pop()
    step.destroy()
    self.showCurStep()
  
  def next(self, event=None):
    step = self.getNextStep()
    if step is None: return
    self.steps[-1].pack_forget()
    self.steps.append(step)
    self.showCurStep()
  
  def end(self, event=None):
    self.root.destroy()
  
  def cancel(self, event=None):
    self.root.destroy()
  

class SplashInfo(Label):
  
  def __init__(self, parent, text):
    self.root = parent
    Label.__init__(self, parent, text=text, background=COLOR_TITLE, foreground="#505099", font="Helvetica 9 bold", border=2, relief=RAISED)
    self.show()
  
  def show(self):
    self.place(relx=0.5, rely=0.5, anchor=CENTER)
    self.root.update_idletasks()
  
  def hide(self):
    self.place_forget()
    self.root.update_idletasks()
    self.destroy()
  

## =====================================================================
## Fonctions utilitaires pour tous les wizards
## =====================================================================

def getWebBouchots(name, pycc):
  """ Cherche les bouchots correspondant  un nom donn parmi les
      bouchots prconfigurs sur le serveur Web
  """
  found_bouchots = []
  error = ""
  try:
    file = open_url(AUTOCONFIG_SERVER, '/', pycc.options, data="bouchot=%s" % name)
    res = file.read()
    file.close()
  except HTTPError, value:
    error = str(value)
  #except:
  #  error = "Erreur http inconnue durant le transfert"
  else:
    found_bouchots = [ line for line in res.split('\n') if line.strip() ]
  return found_bouchots, error

def getWebConfig(filename, pycc):
  """ Rapatrie un fichier de config donn  partir du serveur Web
  """
  res = error = ""
  try:
    file = open_url(AUTOCONFIG_SERVER, '/%s.xml' % filename, pycc.options)
    res = file.read()
    file.close()
  except HTTPError, value:
    error = str(value)
  #except:
  #  error = "Erreur http inconnue durant le transfert"
  return res, error
  
def checkBouchotName(name, parent):
  """ Vrifie la validit d'un nom de bouchot
  """
  if not name:
    tkMessageBox.showwarning("Nouveau bouchot", "Vous devez saisir un nom", parent=parent)
    return 0
  for ch in name:
    if ch.lower() not in "abcdefghijklmnopqrstuvwxyz0123456789_-":
      tkMessageBox.showwarning("Nouveau bouchot", "Seuls les caractres alphanumriques, '_' et '-'\nsont autoriss pour les noms de bouchots", parent=parent)
      return 0
  if name in get_available_bouchots(parent.wiz.pycc.config_path):
    tkMessageBox.showwarning("Nouveau bouchot", "Le bouchot '%s' existe dj, utiliser un autre nom" % name, parent=parent)
    return 0
  return 1


## =====================================================================
## Assistant au premier lancement de pycc
## =====================================================================

class FRWIntro(WizardStep):
  
  def body(self):
    self.title = "pyCoinCoin %s - Premier lancement" % PYCC_VERSION
    self.info  = "\_o< ~ Bienvenue dans pyCoinCoin ! ~ >o_/\n\n" + \
        "Cet assistant vous permet d'effectuer les premiers rglages ncessaires au fonctionnement de votre agent de moulage.\n\n" + \
        "Laissez-vous guider dans les tapes qui suivent, afin d'obtenir un canard  l'oeil soyeux et au poil vif apte  satisfaire les exigences du plus press des dcideurs."
    self.tags = { 'bold': [(0,42)], 'blue': [(0,42)], 'hilight': [(0,42)] }
  
  def validate(self):
    self.wiz.create_config()
    self.next = FRWIdentity(self.wiz)
    return 1

class FRWIdentity(WizardStep):
  
  def body(self):
    self.title = "Cration d'identit"
    self.info  = "pyCoinCoin 0.8 vous permet de grer plusieurs identits. " + \
      "Basiquement, une identit consiste en un pseudo et un user-agent, auxquels on peut adjoindre " + \
      "les cookies d'authentification des bouchots.\nMme si vous moulez de manire anonyme, il est " + \
      "ncessaire de crer au moins une identit avec pseudo, mais dans ce cas, seul le user-agent sera pris en compte.\n\n" + \
      "Crez maintenant votre identit principale."
    self.w_login = TextField(self, "Login ou pseudo")
    self.w_login.pack(anchor=E, padx=10)
    self.w_ua = TextField(self, "User-agent", "pyCoinCoin/%s" % PYCC_VERSION)
    self.w_ua.pack(anchor=E, padx=10)
    return self.w_login
  
  def validate(self):
    login = self.w_login.get()
    if not login:
      tkMessageBox.showwarning(self.title, "Le champ login/pseudo est obligatoire !", parent=self)
      self.w_login.focus_set()
      return 0
    self.wiz.identities.eraseAll()
    self.wiz.identities.add(login, Identity(login, self.w_ua.get()))
    self.next = FRWResal(self.wiz)
    return 1

class FRWResal(WizardStep):
  
  def body(self):
    self.title = "Paramtres rseau"
    self.info  = "Si vous utilisez un proxy pour les connections http, renseignez-le ici. Attention de respecter la syntaxe suivante :\n" + \
      "    http://le.serveur:port/\n\n" + \
      "Dans le cas d'un proxy avec authentification, prcisez le nom d'utilisateur et le mot de passe sous cette forme :\n" + \
      "    mon_login:mon_password"
    self.tags = { 'bold': [(121,146), (260,286)], 'red': [(121,128), (138,139), (143,144), (273,274)], 'blue': [(128,138), (139,143), (260,273), (274,286)] }
    self.w_proxy = TextField(self, "Proxy (adresse:port)")
    self.w_proxy.pack(anchor=E, padx=6)
    self.w_auth = TextField(self, "Authentification proxy (login:passwd)")
    self.w_auth.pack(anchor=E, padx=6)
    return self.w_proxy
  
  def validate(self):
    self.wiz.config.set('http_proxy', self.w_proxy.get())
    self.wiz.config.set('http_proxy_auth', self.w_auth.get())
    self.next = FRWBrowser(self.wiz)
    return 1
  
class FRWBrowser(WizardStep):
  
  def body(self):
    self.title = "Navigateur Web"
    self.info  = "Entrez ici la commande que vous voulez utiliser pour ouvrir des URLs depuis pyCoinCoin.\n\n" + \
      "Attention, il faut donner le path complet (absolu).\n" + \
      "Vous pouvez utiliser la squence spciale '%url' pour dsigner l'url dans la commande."
    self.w_browser = FileField(self, "Browser web")
    self.w_browser.pack()
    return self.w_browser
  
  def validate(self):
    self.wiz.config.set('custom_browser', self.w_browser.get())
    self.next = FRWServices(self.wiz)
    return 1

class FRWServices(WizardStep):
  
  def body(self):
    self.title = "Services  activer"
    self.info  = "PyCoinCoin vous propose d'activer les services suivants par dfaut sur les tribunes. " + \
      "D-slectionnez ceux que vous ne dsirez pas activer.\n\n" + \
      "Ceci n'a rien de dfinitif : par la suite, ces services pourront tre ractivs " + \
      "ou configurs diffremment (voir la section \"PostProcessor\" dans le Centre de " + \
      "Contrle de pyCoinCoin)."
    services = { \
      'newposts': "Mise en surbrillance des nouveaux posts",
      'myposts':  "Reconnaissance de mes posts",
      'reponse':  "Reconnaissance des rponses  mes posts",
      'bigorno':  "Bigornophone",
      'trollo':   "Calcul du trollo-score",
      'smiley':   "Smileys graphiques ala hfr",
      'miniurl':  "Transformation des [url]"
    }
    self.w_srv = {}
    for s, descr in services.items():
      self.w_srv[s] = BoolField(self, descr, 1)
      self.w_srv[s].pack(fill=X)
    return None
  
  def validate(self):
    for s in [ 'trollo', 'miniurl', 'smiley', 'reponse', 'myposts', 'newposts', 'bigorno' ]:  # L'ordre est important
      if self.w_srv[s].get():
        if s == 'newposts':
          self.wiz.pp.add_filter(Filter("Nouveaux posts", [ActionSetStyle('newposts')]))
        elif s == 'myposts':
          self.wiz.pp.add_filter(Filter("Mes posts de moi", [ActionSetStyle('myposts', 'time')], ConditionSet([Condition('login', OP_ONE_ID)])))
        elif s == 'reponse':
          self.wiz.pp.add_filter(Filter("Rponses  mes posts", [ActionSetStyle('answer', 'time'), ActionBlink()], ConditionSet([Condition('message', OP_HAS_REF_TO, cset=ConditionSet([Condition('login', OP_ONE_ID)]))])))
        elif s == 'smiley':
          self.wiz.pp.add_filter(Filter("Smileys ala HFR", [ActionPlugin('TotozSmileys')]))
        elif s == 'trollo':
          self.wiz.pp.add_filter(Filter("Trollomtre", [ActionPlugin('trollo')], after_pinni=0))
        elif s == 'bigorno':
          mylogins = "|".join([id.login for id in self.wiz.identities.get().values()])
          self.wiz.pp.add_filter(Filter("Bigornophone", [ActionMessage("%l appelle sur %h@%t",0,1), ActionBlink("#00aa00")], ConditionSet([Condition('message', OP_MATCHES, "(%s|moules)[<]" % mylogins)])))
        elif s == 'miniurl':
          self.wiz.pp.add_filter(Filter("Mini-urls", [ActionPlugin('urltransfo')], after_pinni=0))
    self.wiz.apply_config()
    self.next = FRWTheEnd(self.wiz)
    return 1

class FRWTheEnd(WizardStep):
  
  def body(self):
    self.title = "Ready To Start A New Mouling eXPerience"
    self.info  = "Voil qui est cool, la config de base est termine !\n\n" + \
      "Cliquez  prsent sur 'Terminer' pour configurer votre premier bouchot."
    self.islast = 1

class FirstRunWizard(Wizard):

  def __init__(self, root, config_path, script_path, isroot=1):
    self.config_path = config_path
    self.script_path = script_path
    image = os.path.join(script_path, "pyccwiz.gif")
    Wizard.__init__(self, root, title="PyCoinCoin First Launch Wizard", image=image, isroot=isroot)
    self.initialize()
  
  def initialize(self):
    self.steps.append(FRWIntro(self))
    self.showCurStep()
  
  def cancel(self, event=None):
    if tkMessageBox.askyesno(self.title, "La configuration n'est pas termine !\nSi vous arrtez maintenant, pyCoinCoin ne pourra tre lanc.\nVoulez-vous quitter ?", parent=self.root):
      self.root.destroy()
    else:
      return
  
  def end(self, event=None):
    self.root.canceled = 0
    self.root.destroy()
  
  def create_config(self):
    log.info("Cration du rpertoire de config")
    try:
      import shutil
      base = os.path.join(self.script_path, "config")
      shutil.copytree(base, self.config_path)
    except:
      log.critical("*** Impossible de copier la config dans le rpertoire utilisateur %s ***" % self.config_path)
      sys.exit(1)
    log.info("Prchargement des options de base")
    self.config = GlobalOptions(path=self.config_path)
    self.identities = Identities(path=self.config_path)
    self.pp = PostProcessor(path=self.config_path)
  
  def apply_config(self):
    self.config.save()
    self.identities.save()
    self.pp.save()


## =====================================================================
## Assistant de configuration d'un nouveau bouchot
## =====================================================================

class BCWIntro(WizardStep):
  
  def body(self):
    self.title = "Nouveau bouchot"
    self.info  = "Bienvenue sur Da Bouchot Config Wizard !\n\n" + \
        "Cet assistant vous permet de configurer pyCoinCoin pour mouler sur un nouveau bouchot.\n\n" + \
        "Tout d'abord, veuillez saisir un nom pour le bouchot (ex. 'dlfp', 'woof', 'f-cpu'...)\n" + \
        "Ce nom sera celui de l'onglet affect au bouchot."
    self.tags = { 'bold': [(0,41)], 'blue': [(0,41)], 'hilight': [(0,41)] }
    self.w_name = TextField(self, "Nom du bouchot")
    self.w_name.pack()
    self.w_lookwebconf = BoolField(self, "Configuration automatique\n(recherche parmi les bouchots prconfigurs sur chrisix.free.fr)", 1)
    self.w_lookwebconf.pack()
    return self.w_name
  
  def validate(self):
    name = self.w_name.get()
    if not checkBouchotName(name, self):
      return 0
    self.wiz.name = name
    if self.w_lookwebconf.get():
      status = SplashInfo(self, "Recherche des bouchots prconfigurs\nTransfert depuis http://%s/ en cours..." % AUTOCONFIG_SERVER)
      self.next = BCWFindBouchot(self.wiz)
      status.hide()
    else:
      self.wiz.manu = 1
      self.next = BCWEnterAddress(self.wiz)
    return 1

class BCWEnterAddress(WizardStep):
  
  def body(self):
    self.title = "Adresse du serveur"
    self.info  = "Entrez ici l'adresse du serveur http sur lequel est hberg le bouchot '"+self.wiz.name+"'.\n\n" + \
      "Entrez cette adresse sous la forme le.serveur.tld (sans 'http://' devant ni '/'  la fin)."
    r1 = 111+len(self.wiz.name)
    r2 = 125+len(self.wiz.name)
    self.tags = { 'bold': [(r1,r2)], 'red': [(r1,r2)] }
    self.w_server = TextField(self, "Adresse du serveur http")
    self.w_server.pack()
    return self.w_server
  
  def validate(self):
    server = self.w_server.get()
    if not server:
      tkMessageBox.showwarning(self.title, "L'adresse du serveur doit tre renseigne !", parent=self)
      return 0
    self.wiz.createBouchot()
    self.wiz.bouchot.options.set('http_server', server)
    self.next = BCWSelectModules(self.wiz)
    return 1

class BCWSelectModules(WizardStep):
  
  def body(self):
    self.title = "Modules  configurer"
    self.info  = "Les types de modules suivants sont disponibles. Slectionnez ceux que vous dsirez configurer.\n\n" + \
      "Notez que sur un bouchot traditionnel, il y a au moins une tribune :) - d'autres modules " + \
      "peuvent tre ajouts par la suite via le Centre de Contrle de PyCoinCoin."
    self.w_mod = {}
    self.w_name = {}
    first = None
    for mod in self.wiz.pycc.modules.keys():
      fr = Frame(self)
      self.w_mod[mod] = BoolField(fr, mod, (mod=='tribune'))
      self.w_mod[mod].pack(side=LEFT)
      self.w_name[mod] = TextField(fr, "Nom :", mod)
      self.w_name[mod].pack(side=RIGHT)
      if first is None: first = self.w_name[mod]
      fr.pack(fill=X, padx=10)
    return first
  
  def validate(self):
    modlist = {}
    for mod, widget in self.w_mod.items():
      if widget.get():
        name = self.w_name[mod].get()
        if not name:
          tkMessageBox.showwarning(self.title, "Vous devez spcifier un nom pour chaque module  configurer !", parent=self)
          return 0
        modlist[name] = mod
    if not modlist.keys():
      if tkMessageBox.askyesno(self.title, "Vous n'avez slectionn aucun module.\nVoulez-vous terminer l'assistant et les configurer plus tard ?", parent=self):
        self.next = BCWSelectId(self.wiz)
        self.next.islast = 1
        return 1
      return 0
    self.wiz.bouchot.options.set('base_modules', modlist)
    self.wiz.bouchot.options.set('startup_modules', modlist.keys())
    self.wiz.createModules(self)
    self.next = BCWSelectId(self.wiz)
    return 1

class BCWFindBouchot(WizardStep):
  
  def body(self):
    self.title = "Bouchot  ajouter"
    name = self.wiz.name
    found_bouchots, error = getWebBouchots(name, self.wiz.pycc)
    self.wiz.manu = 0
    if error:
      self.info = "Configuration automatique impossible, problme de connexion avec le serveur '" + AUTOCONFIG_SERVER + \
        "' (dtails de l'erreur ci-dessous).\n\nPour configurer manuellement ce bouchot, cliquez" + \
        " sur 'Suivant', ou revenez en arrire pour r-essayer."
      Label(self, text=error, foreground="#FF6060").pack()
      self.wiz.manu = 1
      return None
    if len(found_bouchots) > 1:
      def but_callback(b):
        self.w_name.set(b)
        self.wiz.webbouchot = b
      self.info = "PyCoinCoin a trouv plusieurs bouchots possibles.\n\n" + \
        "Slectionnez le bouchot que vous voulez configurer. Vous pouvez" + \
        " changer le nom propos par dfaut avant de cliquer sur 'Suivant'."
      self.w_name = TextField(self, "Nom du bouchot", name)
      self.radiobuttons = Pmw.RadioSelect(self, buttontype='radiobutton', orient='vertical', command=but_callback)
      self.radiobuttons.pack()
      self.w_name.pack(pady=4)
      for bouchot in found_bouchots:
        name, descr, address = bouchot.split('|')
        self.radiobuttons.add(name, text="%s : %s\nsur %s" % (name, descr, address), justify=LEFT, anchor=W)
      self.radiobuttons.invoke(0)
      return self.w_name
    if found_bouchots:
      self.info = "PyCoinCoin a trouv le bouchot ci-dessous.\n\n" + \
        "Vous pouvez changer le nom propos par dfaut si vous le dsirez, " + \
        "puis cliquez sur 'Suivant' pour dmarrer la configuration."
      name, descr, address = found_bouchots[0].split('|')
      self.wiz.webbouchot = name
      Label(self, text="Bouchot trouv :").pack()
      Label(self, text="%s : %s" % (name, descr), font="Helvetica 10 bold", foreground="#409940").pack()
      Label(self, text="sur %s" % address).pack()
      self.w_name = TextField(self, "Nom du bouchot", name)
      self.w_name.pack(pady=4)
      return self.w_name
    self.info = "Aucun bouchot n'a t trouv correspondant  '" + name + "'.\n\n" + \
      "Vous pouvez cliquer sur 'Suivant' pour configurer manuellement ce bouchot," + \
      " ou revenir en arrire pour r-essayer."
    self.wiz.manu = 1
    return None
  
  def validate(self):
    config = ""
    if self.wiz.manu:
      self.next = BCWEnterAddress(self.wiz)
      return 1
    name = self.w_name.get()
    if not checkBouchotName(name, self):
      return 0
    status = SplashInfo(self, "Rcupration du fichier de configuration de '%s'\ndepuis http://%s/ en cours..." % (self.wiz.webbouchot, AUTOCONFIG_SERVER))
    self.wiz.name = name
    config, error = getWebConfig(self.wiz.webbouchot, self.wiz.pycc)
    status.hide()
    if error:
      self.next = BCWGetConfigError(self.wiz, error, 1)
      return 1
    self.wiz.createBouchot(config)
    self.wiz.createModules(self)
    return 1

class BCWGetConfigError(WizardStep):
  
  def __init__(self, wiz, error, tocreate=0):
    self.error = error
    self.tocreate = tocreate
    WizardStep.__init__(self, wiz)
  
  def body(self):
    self.title = "Erreur"
    self.info  = "Une erreur est survenue pendant la tentative de rcupration des " + \
      "paramtres de configuration sur le serveur " + AUTOCONFIG_SERVER + \
      " (dtails de l'erreur ci-dessous.\n\nPour continuer  configurer ce bouchot" + \
      " manuellement, cliquez sur 'Suivant', ou revenez en arrire pour r-essayer."
    Label(self, text=self.error, foreground="#FF6060").pack()
    return None
  
  def validate(self):
    self.wiz.manu = 1
    if self.tocreate:
      self.wiz.createBouchot()
    return 1

class BCWSelectId(WizardStep):
  
  def body(self):
    self.title = "Identit"
    self.info = "Slectionnez l'identit qui sera associe par dfaut avec ce bouchot.\n\n" + \
      "Si le pseudo que vous utilisez n'est pas disponible, crez une nouvelle identit."
    self.idsel = ComboField(self, "Identit:", self.wiz.pycc.identities.get().keys())
    self.idsel.pack()
    Button(self, text="Nouvelle identit...", command=self.new_id).pack(side=RIGHT)
    return self.idsel
  
  def new_id(self, event=None):
    res = tkSimpleDialog.askstring(self.title, "Login/Pseudo:", parent=self)
    if res is not None:
      ident = Identity(res)
      self.wiz.pycc.identities.add(res, ident)
      self.wiz.pycc.apply_config('cident')
      newlist = self.wiz.pycc.identities.get().keys()
      self.idsel.setitems(newlist)
      self.idsel.set(res)
  
  def validate(self):
    self.wiz.bouchot.options.set('default_id', self.idsel.get())
    if self.wiz.bouchot.trib is None:
      self.next = BCWOtherModules(self.wiz)
    else:
      self.next = BCWSetCookie(self.wiz)
    return 1

class BCWSetCookie(WizardStep):
  
  def body(self):
    self.title = "Cookie d'authentification"
    self.info  = "Entrez ici votre cookie d'authentification pour le bouchot "+self.wiz.name+"."+ \
      "\nVous pouvez obtenir automatiquement un nouveau cookie en cliquant sur le bouton de login."+ \
      "\n\nSi vous ne dsirez pas y mouler authentifi, cochez la case approprie."
    Button(self, text="Se loguer sur le bouchot...", command=self.logon_bouchot).pack(anchor="e")
    self.w_cookie = TextField(self, "Cookie", self.wiz.bouchot.trib.options.val('cookie_scheme'))
    self.w_cookie.pack()
    self.w_anonyme = BoolField(self, "Je suis anonyme sur ce bouchot")
    self.w_anonyme.pack(pady=6)
    return self.w_cookie
  
  def logon_bouchot(self):
    if not self.wiz.bouchot.trib.options.val('login_url'):
      if tkMessageBox.askyesno(self.title, "L'URL de login sur ce bouchot n'a pas t spcifie.\nConfigurer maintenant les paramtres de login ?", parent=self):
        OptionsDialog("Configuration de l'authentification", self, self.wiz.bouchot.trib.options, extra_arg=self.wiz.pycc)
      else:
        return
    id = self.wiz.bouchot.options.val('default_id')
    LoginDialog(self, self.wiz.pycc, self.wiz.name, id, force=1)
    try:
      self.w_cookie.set(self.wiz.pycc.identities.get(id).cookies[self.wiz.name])
    except:
      pass
  
  def validate(self):
    mandatory_params = [ 'backend', 'add_post_url', 'add_post_data', 'slip_style' ]
    params = [p for p in mandatory_params if not self.wiz.bouchot.trib.options.val(p)]
    if not params:
      self.next = BCWSelectTheme(self.wiz)
    else:
      self.next = BCWTribParams(self.wiz, params)
    if self.w_anonyme.get():
      return 1
    cookie = self.w_cookie.get()
    cvals = cookie.split('=')
    if len(cvals) == 2:
      nonvide = len(cvals[-1])
    else:
      nonvide = (len(cvals) > 2)
    if not cookie or not nonvide:
      return tkMessageBox.askyesno(self.title, \
        "Vous n'avez pas rempli le cookie. Voulez-vous rester anonyme sur ce bouchot ?", parent=self)
    id = self.wiz.pycc.identities.get(self.wiz.bouchot.options.val('default_id'))
    id.cookies[self.wiz.name] = cookie
    return 1

class BCWTribParams(WizardStep):
  
  def __init__(self, wiz, params):
    self.params = params
    WizardStep.__init__(self, wiz)
  
  def body(self):
    self.title = "Paramtres obligatoires"
    if self.wiz.manu:
      self.info = "Les paramtres suivants doivent tre renseigns pour pouvoir mouler sur le nouveau bouchot."
    else:
      self.info = "Les paramtres suivants sont obligatoires et n'ont pas t trouvs dans le fichier de configuration automatique, veuillez les renseigner."
    self.info += "\n\nEn cas d'erreur, pas de panique, vous pourrez toujours " + \
      "par la suite les changer via le Centre de Contrle de PyCoinCoin."
    self.w_opt = {}
    for p in self.params:
      opt = self.wiz.bouchot.trib.options.get(p)
      self.w_opt[p] = TextField(self, opt.descr)
      self.w_opt[p].pack(fill=X)
    return self.w_opt[self.params[0]]
  
  def validate(self):
    for p, widget in self.w_opt.items():
      if not widget.get():
        if not tkMessageBox.askyesno(self.title, "Certains paramtres n'ont pas t renseigns.\nCa risque de marcher beaucoup moins bien,\nvoulez-vous tout de mme continuer ?"):
          return 0
    for p, widget in self.w_opt.items():
      self.wiz.bouchot.trib.options.set(p, widget.get())
    self.next = BCWSelectTheme(self.wiz)
    return 1

class BCWSelectTheme(WizardStep):
  
  def body(self):
    self.title = "Choix de l'apparence"
    self.info  = "Choisissez un thme (ensemble de styles) pour l'apparence du Pinnipde Teletype.\n\n" + \
      "Dans la section \"Apparence\" du Centre de Contrle de PyCoinCoin, vous pourrez rgler plus " + \
      "finement les diffrents styles. De mme, des thmes peuvent y tre ajouts, dits, supprims..."
    self.theme_path = os.path.join(self.wiz.pycc.config_path, 'themes', 'pinni')
    theme_list = [fich[:-4] for fich in os.listdir(self.theme_path) if fich[-4:] == ".xml"]
    self.themes = {}
    for name in theme_list:
      theme = Theme(name, path=self.theme_path)
      if theme.load():
        self.themes[name] = theme
    self.w_theme = ComboField(self, "Thme : ", ['<Apparence par dfaut>']+self.themes.keys())
    self.w_theme.pack()
    self.w_theme.bind_onchange(self.load_theme)
    Label(self, text="Aperu :", justify=LEFT, anchor=W).pack(fill=X)
    self.apercu = Text(self, wrap=WORD, height=8, width=40)
    self.fill_apercu()
    self.apercu.pack()
    self.w_theme.set('<Apparence par dfaut>')
    self.load_theme()
  
  def load_theme(self):
    cur_theme = self.w_theme.get()
    if cur_theme == '<Apparence par dfaut>':
      apply_styles(self.apercu, self.wiz.pycc.default_styles)
    else:
      apply_styles(self.apercu, self.themes[cur_theme])
  
  def fill_apercu(self):
    text = "15:40:23 gruik Plop les moules<\n" + \
           "15:42:42 IE6/WinXP 15:40:23 hi kevin sa fartent ? Arf trop trop fort [url] ! Hein quoi c'est dj pass ? :)\n" + \
           "15:45:00 monlogin 14:00@dlfp plodocus ?\n" + \
           "15:46:10 UnQuiRepond 15:45 [url] fortune detected || et parfois aussi, ton clavier se blo"
    tags = { 'time':  [(0,8), (32,40), (141,149), (181,189)],
             'login': [(9,14), (150,159), (190,201)],
             'info':  [(41,50)],
             'a':          [(101,106), (208,213)],
             'b':          [(101,106), (208,213)],
             'urlvisit':   [(101,106)],
             'urlhilight': [],
             'clock':      [(51,59), (202,207)],
             'crossclock': [(159,169)],
             'hilightref': [(51,59)],
             'tt':      [(214,230)],
             'hilight': [(0,32)],
             'myposts': [(141,149)],
             'answer':  [(181,189)] }
    self.apercu.config(state='normal')
    self.apercu.delete(0.0, 'end')
    self.apercu.insert('end', text)
    for tag, tagranges in tags.items():
      for start, end in tagranges:
        self.apercu.tag_add(tag, "0.0+%dc" % start, "0.0+%dc" % end)
    self.apercu.config(state='disabled')
  
  def validate(self):
    cur_theme = self.w_theme.get()
    if cur_theme == '<Apparence par dfaut>':
      self.wiz.bouchot.styles._objects.update(self.wiz.pycc.default_styles._objects)
    else:
      self.themes[cur_theme].copy_styles_to(self.wiz.bouchot.styles)
    self.next = BCWOtherModules(self.wiz)
    return 1

class BCWOtherModules(WizardStep):

  def body(self):
    self.title = "Modules  activer"
    self.info  = "Les modules que vous slectionnez ici seront activs ds l'ouverture " + \
      "du bouchot.\n\nLe bouton Configuration Avance vous permet d'accder  des " + \
      "paramtres supplmentaires via le Centre de Contrle de PyCoinCoin - " + \
      "vous pouvez l'utiliser en particulier pour les modules non encore configurs."
    self.tags = { 'bold': [(92,113)], 'blue': [(92,113)] }
    self.w_mod = {}
    toactive = self.wiz.bouchot.options.val('startup_modules')
    for mod in self.wiz.bouchot.options.val('base_modules').keys():
      fr = Frame(self)
      self.w_mod[mod] = BoolField(fr,"%s: %s" % (mod, self.wiz.bouchot.modules[mod].options.val('descr')), mod == 'tribune' or mod in toactive)
      self.w_mod[mod].pack(side=LEFT)
      Button(fr, text="Config avance...",
        command=lambda s=self, m=mod: s.wiz.pycc.showConfig("%s_%s" % (s.wiz.name, m), s)).pack(side=RIGHT)
      fr.pack(fill=X)
    self.wiz.save()   # Ici sauvegarde de la config car pyc4 lit depuis les fichiers
    return None
  
  def validate(self):
    toactive = []
    for mod, widget in self.w_mod.items():
      if widget.get():
        toactive.append(mod)
    if not toactive:
      if not tkMessageBox.askyesno(self.title, "Vous n'avez slectionn aucun module.\n" + \
        "Cela signifie que l'onglet du bouchot restera vide au dmarrage.\n" + \
        "Voulez-vous tout de mme continuer ?", parent=self):
        return 0
    self.wiz.bouchot.options.set('startup_modules', toactive)
    self.next = BCWTheEnd(self.wiz, 1)
    return 1

class BCWTheEnd(WizardStep):
  
  def body(self):
    self.title = "\o/ Ay ! \o/"
    self.info  = "Flicitations !\n\nVotre nouveau bouchot est configur.\n\n" + \
      "Cliquez sur 'Terminer' maintenant pour commencer  en profiter. Sinon," + \
      " un clic sur 'Prcdent' vous permettra de modifier encore les paramtres."
    self.tags = { 'bold': [(0,16)], 'blue': [(0,16)], 'hilight': [(0,16)] }
    self.w_go = BoolField(self, "Ouvrir tout de suite mon bouchot tout neuf", 1)
    self.w_go.pack()
  
  def validate(self):
    if self.w_go.get():
      self.wiz.pycc.load_bouchot(self.wiz.name)
      self.wiz.pycc.select_tab(self.wiz.name)
    if self.wiz.pycc.pycccc is not None:
      self.wiz.pycc.pycccc.tree.add_child('cbou', self.wiz.pycc.pycccc.new_bouchot_node(self.wiz.name))
      self.wiz.pycc.pycccc.tree.select(self.wiz.name)
    return 1

class BouchotConfigWizard(Wizard):
  
  def __init__(self, pycc, root, isroot=0):
    self.pycc = pycc
    self.manu = 0
    self.name = self.webbouchot = ""
    image = os.path.join(pycc.script_path, "pyccwiz.gif")
    Wizard.__init__(self, root, title="Da Bouchot Config Wizard", image=image, isroot=isroot)
    self.initialize()
  
  def initialize(self):
    self.steps.append(BCWIntro(self))
    self.showCurStep()
  
  def cancel(self, event=None):
    if tkMessageBox.askyesno(self.title, "La configuration du bouchot n'est pas termine !\nVoulez-vous vraiment quitter sans enregistrer les modifications en cours ?", parent=self.root):
      self.root.destroy()
    else:
      return
  
  def end(self, event=None):
    self.steps[-1].validate()
    self.save()
    self.root.destroy()
  
  def save(self):
    self.bouchot.save_config()
  
  def createBouchot(self, config=''):
    path=os.path.join(self.pycc.config_path, "bouchots", self.name)
    os.mkdir(path)
    if config:
      try:
        file = open(os.path.join(path, "Config.xml"), "w")
        file.write(config)
        file.close()
      except:
        log.error("Bouchot '%s': Erreur pendant l'criture du fichier de config" % self.name)
    self.pycc.add_bouchot(self.name)
    self.bouchot = self.pycc.bouchots[self.name]
  
  def createModules(self, step):
    modules = self.bouchot.options.val('base_modules')
    step.next = None
    for mod, modtype in modules.items():
      if not self.manu:
        status = SplashInfo(step, "Rcupration des modules :\n%s" % mod)
        config, error = getWebConfig("%s_%s" % (self.webbouchot, mod), self.pycc)
        status.hide()
        if error:
          step.next = BCWGetConfigError(self, error)
      else:
        config = ''
      self.bouchot.new_module(mod, modtype)
      if config:
        try:
          file = open(os.path.join(self.bouchot.config_path, mod, "Config.xml"), "w")
          file.write(config)
          file.close()
        except:
          log.error("Bouchot '%s': Erreur pendant l'criture du fichier de config du module '%s'" % (self.name, mod))
      self.bouchot.load_module(mod)
    if step.next is None:
      step.next = BCWSelectId(self)


## =====================================================================
## Assistant de mise  jour d'un bouchot
## =====================================================================

class BUWIntro(WizardStep):

  def body(self):
    self.title = "Mise  jour d'un bouchot"
    self.info  = "Bienvenue sur Da Bouchot Update Wizard !\n\n" + \
      "Cet assistant vous permet de mettre  jour automatiquement vos bouchots grce au " + \
      "serveur de configuration automatique "+AUTOCONFIG_SERVER+".\n\n" + \
      "Pour rechercher maintenant les mises  jour du bouchot '"+self.wiz.name+"', cliquez sur 'Suivant'."
    self.tags = { 'bold': [(0,41)], 'blue': [(0,41)], 'hilight': [(0,41)] }
    return None
  
  def validate(self):
    name = self.wiz.name
    status = SplashInfo(self, "Recherche des bouchots prconfigurs\nTransfert depuis http://%s/ en cours..." % AUTOCONFIG_SERVER)
    found_bouchots, error = getWebBouchots(name, self.wiz.pycc)
    status.hide()
    if error:
      self.next = BUWGetConfigError(self.wiz, error)
      return 1
    if not found_bouchots:
      self.next = BUWNotFound(self.wiz)
      return 1
    toupdate = ""
    for bouchot in found_bouchots:
      bname, descr, address = bouchot.split('|')
      if address == self.wiz.bouchot.options.val('http_server'):
        toupdate = bname
        break
    if not toupdate:
      self.next = BUWSelectMulti(self.wiz, found_bouchots)
      return 1
    self.wiz.webbouchot = toupdate
    self.wiz.getConfig(self)
    return 1

class BUWSelectMulti(WizardStep):

  def __init__(self, wiz, blist):
    self.blist = blist
    WizardStep.__init__(self, wiz)
  
  def body(self):
    self.title = "Mise  jour d'un bouchot"
    self.info = "Le serveur de mises  jour n'a pas trouv de bouchot dont l'adresse du serveur " + \
      "http correspond  votre bouchot '"+self.wiz.name+"'.\n\nEn revanche, les bouchots suivants ont t " + \
      "trouvs sur des serveurs diffrents. Votre bouchot a peut-tre migr de serveur. Si vous " + \
      "pensez que votre bouchot se trouve dans cette liste, slectionnez-le et continuez."
    self.bname = ''
    def rb_callback(b):
      self.bname = b
    self.radiobuttons = Pmw.RadioSelect(self, buttontype='radiobutton', orient='vertical', command=rb_callback)
    self.radiobuttons.pack()
    for bouchot in self.blist:
      bname, descr, address = bouchot.split('|')
      self.radiobuttons.add(bname, text="%s : %s\nsur %s" % (bname, descr, address), justify=LEFT, anchor=W)
    self.radiobuttons.invoke(0)
    return None
  
  def validate(self):
    self.wiz.webbouchot = self.bname
    self.wiz.getConfig(self)
    return 1

class BUWNotFound(WizardStep):
  
  def body(self):
    self.title = "Bouchot inconnu"
    self.info  = "Mise  jour automatique impossible : le bouchot '" + self.wiz.name + \
      "' n'a pas t trouv dans la liste des bouchots pr-configurs."
    self.islast = 1
    return None

class BUWGetConfigError(WizardStep):
  
  def __init__(self, wiz, error):
    self.error = error
    WizardStep.__init__(self, wiz)
  
  def body(self):
    self.title = "Erreur"
    self.info  = "Mise  jour automatique du bouchot '" + self.wiz.name + "' impossible : problme " + \
      "de connexion avec le serveur (voir les dtails de l'erreur ci-dessous).\n\n" + \
      "Vous pouvez cliquer sur 'Prcdent' pour faire une nouvelle tentative."
    Label(self, text=self.error, foreground="#FF6060").pack()
    self.islast = 1
    return None

class BUWShowDelta(WizardStep):
  
  def __init__(self, wiz, title, optset, new_opts, changed_opts, init_modules=0):
    self.optset = optset
    self.new_opts = new_opts
    self.changed_opts = changed_opts
    self.init_modules = init_modules
    wiz.haschanged = 1   # Si on passe par cette tape c'est que des champs ont chang
    WizardStep.__init__(self, wiz, title=title)
  
  def body(self):
    self.info = "PyCoinCoin a dtect les mises  jour suivantes dans les "+self.title+".\n\n" + \
      "Veuillez vrifier les changements que vous voulez appliquer, puis cliquer sur " + \
      "'Suivant' pour continuer."
    self.cvar = {}
    self.w_opt = {}
    first = None
    for name, val in self.changed_opts.items():
      oldval, newopt = val
      #print "--", name
      if hasattr(newopt, 'tohide') or name == 'base_modules':
        Label(self, text=newopt.descr, font="Helvetica 9 bold").pack(anchor=W)
      else:
        fr = Frame(self, border=2, relief=GROOVE)
        Label(fr, text=newopt.descr, font="Helvetica 9 bold").pack(anchor=W)
        self.cvar[name] = IntVar()
        fr1 = Frame(fr)
        Radiobutton(fr1, text="Ancienne valeur:", variable=self.cvar[name], value=0).pack(side=LEFT, anchor=W)
        Label(fr1, text=oldval, foreground="#4040C0").pack(side=LEFT)
        fr1.pack(padx=4, fill=X)
        fr2 = Frame(fr)
        Radiobutton(fr2, text="Nouvelle valeur:", variable=self.cvar[name], value=1).pack(side=LEFT, anchor=W)
        self.w_opt[name] = TextField(fr2, "", newopt.get())
        self.w_opt[name].pack(side=LEFT)
        if first is None: first = self.w_opt[name]
        fr2.pack(padx=4, fill=X)
        self.cvar[name].set(1)
        fr.pack(fill=X)
    labelset = 0
    for name, opt in self.new_opts.items():
      if not labelset:
        Label(self, text="Nouvelles options", font="Helvetica 9 bold", background="#FFFCDA", justify=LEFT, anchor=W).pack(fill=X, anchor=W)
        labelset = 1
      if hasattr(opt, 'tohide'):
        Label(self, text=opt.descr, font="Helvetica 9 bold").pack(anchor=W)
      else:
        self.w_opt[name] = TextField(self, opt.descr, opt.get())
        self.w_opt[name].pack(fill=X)
        if first is None: first = self.w_opt[name]
    return first
  
  def validate(self):
    for name in self.changed_opts.keys():
      # On dtruit puis recre l'option avec le nouvel objet
      self.optset.delete(name)
      newopt = self.changed_opts[name][1]
      self.optset.add(name, newopt)
      if hasattr(newopt, 'tohide') or name == 'base_modules':
        continue  # Si option cache on a fini
      if self.cvar[name].get():
        # Nouvelle valeur de l'option = valeur du champ
        self.optset.set(name, self.w_opt[name].get())
      else:
        # Restitution de l'ancienne valeur
        self.optset.set(name, self.changed_opts[name][0])
    for name, newopt in self.new_opts.items():
      if not hasattr(newopt, 'tohide'):
        # Nouvelle option non cache : on prend la valeur du champ
        newopt.set(self.w_opt[name].get())
        # On l'ajoute dans les nouvelles options  diter
        if not self.optset.exists('new_toedit'):
          self.optset.add('new_toedit', Option("Nouvelles ditables", {}))
        toedit = self.optset.val('new_toedit')
        try:
          toedit[newopt.toedit].append(name)
        except:
          toedit[newopt.toedit] = [name]
      self.optset.add(name, newopt)
    self.optset.addNewEditables()
    self.optset.save()
    if self.init_modules:
      self.wiz.getModules(self)
      return 1
    if self.wiz.updmods:
      if self.wiz.getNextModuleConfig(self):
        return 1
    if self.wiz.newmods:
      self.wiz.createModules(self)
      return 1
    self.next = BUWTheEnd(self.wiz)
    return 1

class BUWNewModules(WizardStep):
  
  def body(self):
    self.wiz.haschanged = 1  # Si nouv modules, c'est que la config a chang
    self.title = "Nouveaux modules"
    self.info  = "Les nouveaux modules suivants ont t trouvs sur le serveur de mises  jour, " + \
      "et ont t ajouts  votre configuration.\n\nPour les activer par dfaut sur le " + \
      "bouchot, cochez la case correspondante. Vous pouvez aussi personnaliser la " + \
      "configuration du module en cliquant sur 'Avanc'."
    self.w_mod = {}
    toactive = self.wiz.bouchot.options.val('startup_modules')
    for mod in self.wiz.newmods:
      fr = Frame(self)
      self.w_mod[mod] = BoolField(fr, mod, mod == 'tribune' or mod in toactive)
      self.w_mod[mod].pack(side=LEFT)
      Button(fr, text="Config avance...",
        command=lambda s=self, m=mod: s.wiz.pycc.showConfig("%s_%s" % (s.wiz.name, mod), self)).pack(side=RIGHT)
      fr.pack(fill=X)
    self.wiz.save()   # Ici sauvegarde de la config car pyc4 lit depuis les fichiers
    return None
  
  def validate(self):
    toactive = self.wiz.bouchot.options.val('startup_modules')
    for mod, widget in self.w_mod.items():
      if widget.get():
        toactive.append(mod)
    self.wiz.bouchot.options.set('startup_modules', toactive)
    self.next = BUWTheEnd(self.wiz, 1)
    return 1

class BUWTheEnd(WizardStep):
  
  def body(self):
    if self.wiz.haschanged:
      self.title = "Mise  jour termine"
      self.info  = "Voil, la mise  jour s'est droule correctement, les options du bouchot '" + \
        self.wiz.name + "' sont  la page, iou are inne ze mouve, c'est trop de la balle !\n\n" + \
        "Un rechargement du bouchot sera peut-tre ncessaire pour que les changements soient " + \
        "pris en compte (notamment si certaines URLs ont chang...)"
    else:
      self.title = "Pas de changements"
      self.info  = "L'assistant de mise  jour n'a pas dtect de changements. Les options du " + \
        "bouchot '" + self.wiz.name + "' sont parfaitement euppe-tou-dyte (a le fait non ?)."
    self.islast = 1
    return None

class BouchotUpdateWizard(Wizard):
  
  def __init__(self, pycc, root, name):
    self.pycc = pycc
    self.name = name
    self.bouchot = self.pycc.bouchots[name]
    self.webbouchot = name
    image = os.path.join(pycc.script_path, "pyccwiz.gif")
    Wizard.__init__(self, root, title="Da Bouchot Update Wizard", image=image, isroot=0)
    self.initialize()
  
  def initialize(self):
    self.haschanged = 0
    self.steps.append(BUWIntro(self))
    self.showCurStep()
  
  def cancel(self, event=None):
    if tkMessageBox.askyesno(self.title, "La mise  jour du bouchot n'est pas termine !\nVoulez-vous vraiment quitter sans enregistrer les modifications en cours ?", parent=self.root):
      self.root.destroy()
    else:
      return
  
  def end(self, event=None):
    self.steps[-1].validate()
    self.save()
    self.root.destroy()
  
  def save(self):
    self.bouchot.save_config()
  
  def getDelta(self, options, newoptions):
    news = {}
    changed = {}
    for optname, opt in newoptions.get().items():
      if hasattr(opt, 'toskip'): continue
      if optname not in options.get().keys():
        news[optname] = opt
        continue
      oldval = options.val(optname)
      if opt.get() != oldval:
        changed[optname] = (oldval, opt)
    return news, changed
  
  def getNewOptions(self, step, info, file):
    status = SplashInfo(step, info)
    config, error = getWebConfig(file, self.pycc)
    status.hide()
    if error:
      step.next = BUWGetConfigError(self, error)
      return None
    if not config.strip():
      step.next = BUWGetConfigError(self, "Fichier de configuration vide")
      return None
    return OptionSet(content=config)
  
  def getConfig(self, step):
    self.newoptions = self.getNewOptions(step, "Rcupration du fichier de configuration de '%s'\ndepuis http://%s/ en cours..." % (self.webbouchot, AUTOCONFIG_SERVER), self.webbouchot)
    if self.newoptions is None: return
    new_opts, changed_opts = self.getDelta(self.bouchot.options, self.newoptions)
    if len(new_opts) or len(changed_opts):
      step.next = BUWShowDelta(self, "Options de bouchot", self.bouchot.options, new_opts, changed_opts, 1)
      return
    self.getModules(step)
  
  def getNextModuleConfig(self, step):
    log.debug("Ici getNextModuleConfig")
    if len(self.updmods) > 0:
      mod = self.updmods.pop()
      log.debug("getNextModuleConfig: module %s" % mod)
      newoptions = self.getNewOptions(step, "Rcupration de la configuration module :\n%s" % mod, "%s_%s" % (self.webbouchot, mod))
      if newoptions is None: return 1
      #print "---- Module", mod, ": il y a des newoptions:", newoptions
      modoptions = self.bouchot.modules[mod].options
      new_opts, changed_opts = self.getDelta(modoptions, newoptions)
      if len(new_opts) or len(changed_opts):
        step.next = BUWShowDelta(self, "Options du module %s" % mod, modoptions, new_opts, changed_opts)
        return 1
      return self.getNextModuleConfig(step)
    return 0
  
  def getModules(self, step):
    log.debug("Ici getModules")
    modlist = self.newoptions.val('base_modules').keys()
    self.newmods = [mod for mod in modlist if mod not in self.bouchot.modules.keys()]
    self.updmods = [mod for mod in modlist if mod in self.bouchot.modules.keys()]
    if self.updmods:
      if self.getNextModuleConfig(step): return
    if self.newmods:
      self.createModules(step)
      return
    step.next = BUWTheEnd(self)
  
  def createModules(self, step):
    log.debug("Ici createModules")
    modules = self.newoptions.val('base_modules')
    for mod, modtype in modules.items():
      if mod not in self.newmods: continue
      status = SplashInfo(step, "Rcupration des modules :\n%s" % mod)
      config, error = getWebConfig("%s_%s" % (self.webbouchot, mod), self.pycc)
      status.hide()
      if error:
        step.next = BUWGetConfigError(self, error)
        return
      if not config:
        step.next = BUWGetConfigError(self, "Fichier de configuration vide")
        return
      self.bouchot.new_module(mod, modtype)
      try:
        file = open(os.path.join(self.bouchot.config_path, mod, "Config.xml"), "w")
        file.write(config)
        file.close()
      except:
        log.error("Bouchot '%s': Erreur pendant l'criture du fichier de config du module '%s'" % (self.name, mod))
      self.bouchot.load_module(mod)
    step.next = BUWNewModules(self)
